/*
 * AndroidPortAdditions.cpp
 *
 *  Created on: Jan 14, 2013
 *      Author: omergilad
 */

#if defined(__ANDROID__)

// Allow use of stuff in <time.h>
#define FORBIDDEN_SYMBOL_EXCEPTION_time_h

// Disable printf override in common/forbidden.h to avoid
// clashes with log.h from the Android SDK.
// That header file uses
//   __attribute__ ((format(printf, 3, 4)))
// which gets messed up by our override mechanism; this could
// be avoided by either changing the Android SDK to use the equally
// legal and valid
//   __attribute__ ((format(printf, 3, 4)))
// or by refining our printf override to use a varadic macro
// (which then wouldn't be portable, though).
// Anyway, for now we just disable the printf override globally
// for the Android port
#define FORBIDDEN_SYMBOL_EXCEPTION_printf

#include "backends/platform/android/android.h"
#include "backends/platform/android/jni.h"

#include "AndroidPortAdditions.h"

#include "CursorAndroidBitmap.h"

#include "graphics/CursorDrawable.h"
#include "graphics/AlphaAnimation.h"
#include "graphics/ScaleAnimation.h"
#include "graphics/WaitForConditionAnimation.h"
#include "graphics/ParallelAnimation.h"

#include "graphics/SequenceAnimationComposite.h"
#include "graphics/RepeatAnimationWrapper.h"
#include "graphics/DeccelerateInterpolator.h"
#include "graphics/AccelerateInterpolator.h"

#include "Constants.h"

#include "engines/groovie/groovie.h"
#include "engines/queen/queen.h"
#include "engines/saga/saga.h"

#include "event_types.h"

using std::shared_ptr;

// Suppress the compiler warning about an (unsigned int >= 0).
// The constant that contains 0 might change later.
#pragma GCC diagnostic ignored "-Wtype-limits"

static inline GLfixed xdiv(int numerator, int denominator) {
	assert(numerator < (1 << 16));
	return (numerator << 16) / denominator;
}

AndroidPortAdditions* AndroidPortAdditions::sInstance = NULL;

AndroidPortAdditions::AndroidPortAdditions() :
mDisplayRatio(0), mRenderMeasureSkippedFrameCount(0), mShouldUseModifiedGamePixels(
		false), mSelectedChatRow(0), mBitmaps(), mGlTextures(NULL), mShouldPerformRevealItems(
		false), mMouseVisible(false), mClassicMode(false), mTouchTranslationOffset(
		0), mGameInChat(false), mDumpPixelsToFile(false), mCurrentAction(
		ACTION_WALK), mDisallowNextTouchEvent(false), mCurrentHotspot(
		Point(0, 0), Point(0, 0), Rect()), mLastTouchMoveTimestamp(0), mBottomToolbarAppearing(
		false), mActionIconFadeCounter(0), mRevealItemsFadeCounter(0), mFirstDrawTimestamp(
		0), mGameInPostcard(false), mSkipIconFadeCounter(0), mGameDisplayStarted(
		false), mAutoLoadSlot(-1), mAutoLoadSpamSkipCounter(0), mShaderTestStartTime(
		0), mAutoLoadIntroEndTimestamp(0), mScalingFactor(2), mLQShaderScalingFactor(
		2.0), mScalingOption(SCALING_OPTION_SOFT), mScaledOutput(NULL), mScalerPlugin(
		NULL), mShouldTestShader(false), mRenderMeasureCount(0), mRenderMeasureAverage(
		0.0), mSkipPressedAtLeastOnce(false), mSlotToLoad(-1), mSlotToSave(
		-1), mLastIdlePreventionTimestamp(0), mGameCancelledIdleWhileSaving(
		false), mSaveStartTimestamp(0), mSaveInProgress(false), mGameIdleCountWhileSaving(
		false), mCurrentGameArea(OTHER), mGameStartedInAutoload(false), mForceSaveLoad(
		false), mAutoLoadState(false), mCurrentHotspotDrawable(NULL), mCurrentHotspotRectDrawable(
		NULL), mCurrentActionDrawable(
		NULL), mShouldPerformSimon1PuddleWorkaround(false), mShaderScalingMaxResolutionW(
		SHADER_SCALING_MAX_RESOLUTION_W), mShaderScalingMaxResolutionH(
		SHADER_SCALING_MAX_RESOLUTION_H), mLastTimeSkipPressed(0), mLastTimeRevealItemsPressed(
		0), mGameType(CURRENT_GAME_TYPE), mAfterGameIntro(false), mDrawTransformationWidth(
		1.0), mDrawTransformationHeight(1.0), mShouldShowNewActionBar(
		true), mPreventFingerCoveringHotspot(true), mLowerButtonPosY(0), mStartInOpenHouse(
		false), mDebugHideIconsToggle(false), mMouseJustTurnedInvisible(
		false), mUseFotaqHotspotWorkaround(true), mUpperTextIsLong(
		false) {

	mEmptyClearingBuffer = new uint16[GL_DYNAMIC_TEXTURE_WIDTH
	* GL_DYNAMIC_TEXTURE_HEIGHT];
	memset(mEmptyClearingBuffer, 0,
			GL_DYNAMIC_TEXTURE_WIDTH * GL_DYNAMIC_TEXTURE_HEIGHT);
}

bool AndroidPortAdditions::shouldUseLongClickAction() {
	return (mLastTouchMoveTimestamp != 0
			&& (AndroidPortUtils::getTimeOfDayMillis() - mLastTouchMoveTimestamp
					>= LONG_CLICK_ACTION_MIN_TOUCH_TIME));
}

void AndroidPortAdditions::clearCurrentActionIconAnimation(
		bool actionPerformed) {

	if (mCurrentActionDrawable == NULL) {
		return;
	}

	if (getGameType() == GAME_TYPE_IHNM) {
		// IHNM: no action icons
		mCurrentActionDrawable = NULL;
		return;
	}

	AnimationPtr actionEndAnimation;
	if (actionPerformed) {

		// Create the initial fade in animation
		shared_ptr<AlphaAnimation> initialFadeInAnimation(new AlphaAnimation());
		initialFadeInAnimation->setDuration(ACTION_BLINK_FADE_DURATION);
		initialFadeInAnimation->setStartAlpha(
				mCurrentActionDrawable->getAlpha());
		initialFadeInAnimation->setEndAlpha(1);

		// Create the loop fade in animation
		shared_ptr<AlphaAnimation> loopFadeInAnimation(new AlphaAnimation());
		loopFadeInAnimation->setDuration(ACTION_BLINK_FADE_DURATION);
		loopFadeInAnimation->setStartAlpha(0);
		loopFadeInAnimation->setEndAlpha(1);

		// Create a fade out animation
		shared_ptr<AlphaAnimation> fadeOutAnimation(new AlphaAnimation());
		fadeOutAnimation->setDuration(ACTION_BLINK_FADE_DURATION);
		fadeOutAnimation->setStartAlpha(1);
		fadeOutAnimation->setEndAlpha(0);

		// Create a sequence for fade in \ fade out
		shared_ptr<SequenceAnimationComposite> fadeSequenceAnimation(
				new SequenceAnimationComposite());
		fadeSequenceAnimation->addAnimation(loopFadeInAnimation);
		fadeSequenceAnimation->addAnimation(fadeOutAnimation);

		// Create a repeatable animation
		// We repeat times - 1 because the first "repeat" is done separately, to consider the starting alpha value
		shared_ptr<RepeatAnimationWrapper> repeatAnimation(
				new RepeatAnimationWrapper(fadeSequenceAnimation,
						ACTION_BLINK_REPEAT_COUNT - 1));

		// Create a sequence of the initial fade in, and the loop
		shared_ptr<SequenceAnimationComposite> actionBlinkAnimation(
				new SequenceAnimationComposite());
		actionBlinkAnimation->addAnimation(initialFadeInAnimation);
		actionBlinkAnimation->addAnimation(fadeOutAnimation);
		actionBlinkAnimation->addAnimation(repeatAnimation);

		actionEndAnimation = actionBlinkAnimation;
	} else {
		// Create a fade out animation
		shared_ptr<AlphaAnimation> fadeOutAnimation(new AlphaAnimation());
		fadeOutAnimation->setDuration(ACTION_LEAVE_FADEOUT_DURATION);
		fadeOutAnimation->setStartAlpha(mCurrentActionDrawable->getAlpha());
		fadeOutAnimation->setEndAlpha(0);
		fadeOutAnimation->setInterpolator(
				InterpolatorPtr(new DeccelerateInterpolator));

		actionEndAnimation = fadeOutAnimation;
	}

	actionEndAnimation->start(AndroidPortUtils::getTimeOfDayMillis());
	mCurrentActionDrawable->setAnimation(actionEndAnimation);
	mCurrentActionDrawable = NULL;
}

bool AndroidPortAdditions::canSkip() {
	if (g_engine == NULL) {
		return false;
	}

	if (getGameType() == GAME_TYPE_FOTAQ) {
		return g_engine->canSkip();
	} else if (getGameType() == GAME_TYPE_IHNM) {
		return g_engine->canSkip();
	} else {
		return (!mMouseVisible && g_engine->canSkip());
	}
}

bool AndroidPortAdditions::canReplay() {
	if (g_engine == NULL) {
		return false;
	}

	return (mGameType == GAME_TYPE_T7G && mMouseVisible && g_engine->canReplay());
}

bool AndroidPortAdditions::canShowMap() {
	if (g_engine == NULL) {
		return false;
	}

	return (mGameType == GAME_TYPE_T7G && mMouseVisible
			&& g_engine->canShowMap());
}

bool AndroidPortAdditions::canShowRevealItems() {

	if (getGameType() == GAME_TYPE_SIMON1 || getGameType() == GAME_TYPE_SIMON2) {
		return (mBottomToolbarAppearing || g_engine->isInScrollingMap())
		&& mMouseVisible;
	} else if (mGameType == GAME_TYPE_T7G) {
		return mMouseVisible;
	} else if (mGameType == GAME_TYPE_FOTAQ || mGameType == GAME_TYPE_IHNM) {
		return mMouseVisible && !mGameInChat;
	}
}

bool AndroidPortAdditions::canShowMenuButton() {

	if (mGameType == GAME_TYPE_SIMON1 || mGameType == GAME_TYPE_SIMON2) {
		return (mBottomToolbarAppearing || mGameInChat
				|| g_engine->isInScrollingMap()) && mMouseVisible;
	} else if (mGameType == GAME_TYPE_T7G) {
		return mMouseVisible;
	} else if (mGameType == GAME_TYPE_FOTAQ) {
		return mMouseVisible && (mBottomToolbarAppearing || mGameInChat);
	} else if (mGameType == GAME_TYPE_IHNM) {
		return (mMouseVisible);
	}
}

uint16 AndroidPortAdditions::getCurrentAction() {
	if (g_engine == NULL) {
		return ACTION_WALK;
	}

	if (getGameType() == GAME_TYPE_T7G) {
		return ACTION_WALK;
	}

	return g_engine->getCurrentActionId();
}

void AndroidPortAdditions::onActionChanged(uint16 action) {
	LOGD("AndroidPortAdditions::onActionChanged: %d", action);
	mCurrentAction = action;
}

void AndroidPortAdditions::onActionClicked(uint16 action) {

//	addBigActionFadeAnimation(getActionIcon(action));
}

void AndroidPortAdditions::addBigActionFadeAnimation(AndroidBitmap* bitmap) {

// Add a fade out animation for the action
	DrawablePtr actionDrawable(new Drawable);
	actionDrawable->setBitmap(bitmap);
	actionDrawable->setPositionX(BIG_ACTION_ICON_X);
	actionDrawable->setPositionY(BIG_ACTION_ICON_Y);
	actionDrawable->setWidth(BIG_ACTION_ICON_W);

	shared_ptr<AlphaAnimation> fadeInAnimation(new AlphaAnimation);
	fadeInAnimation->setDuration(BIG_ACTION_ICON_FADEIN_DURATION);
	fadeInAnimation->setStartAlpha(0);
	fadeInAnimation->setEndAlpha(BIG_ACTION_ICON_FADE_MAX_ALPHA);
	fadeInAnimation->setInterpolator(
			InterpolatorPtr(new DeccelerateInterpolator));

	shared_ptr<AlphaAnimation> fadeOutAnimation(new AlphaAnimation);
	fadeOutAnimation->setDuration(BIG_ACTION_ICON_FADEOUT_DURATION);
	fadeOutAnimation->setStartAlpha(BIG_ACTION_ICON_FADE_MAX_ALPHA);
	fadeOutAnimation->setEndAlpha(0);
	fadeOutAnimation->setInterpolator(
			InterpolatorPtr(new AccelerateInterpolator));

	shared_ptr<SequenceAnimationComposite> actionAnimation(
			new SequenceAnimationComposite);
	actionAnimation->addAnimation(fadeInAnimation);
	actionAnimation->addAnimation(fadeOutAnimation);

// Add to animation list
	actionAnimation->start(AndroidPortUtils::getTimeOfDayMillis());
	actionDrawable->setAnimation(AnimationPtr(actionAnimation));
	mAnimatedDrawables.push_back(actionDrawable);
}

AndroidPortAdditions::~AndroidPortAdditions() {
	LOGD("AndroidPortAdditions::~AndroidPortAdditions()");

// Delete all the Android bitmap pointers
	for (BitmapHashMap::iterator x = mBitmaps.begin(); x != mBitmaps.end();
			++x) {

		AndroidBitmap* bitmap = x->_value;
		delete bitmap;
	}

	mBitmaps.clear();

	GLCALL(glDeleteTextures(2, mGlTextures));
	delete[] mGlTextures;
	mGlTextures = NULL;

	mModifiedGameSurface.free();

	delete mScaledOutput;

	mScalerPlugin->deinitialize();
	delete mScalerPlugin;

	if (mScalerShaderProgram != mDefaultShaderProgram) {
		delete mScalerShaderProgram;
	}

	delete mDefaultShaderProgram;
	delete mBlackShaderProgram;

	if (getGameType() == GAME_TYPE_T7G) {
		delete mRectHighlightingShaderProgram;
	}

	delete mTouchEventMutex;

	delete mCursorBitmap;
	delete mDummyBitmap;

	delete[] mEmptyClearingBuffer;
}

void AndroidPortAdditions::onSystemInitialized() {
// Mutex depends on the system object
	mTouchEventMutex = new Common::Mutex();

}

void AndroidPortAdditions::onEngineInitialized() {
// This action depends on game engine
	if (mGameType == GAME_TYPE_T7G && mStartInOpenHouse) {
		// Specific for T7G
		((Groovie::GroovieEngine*) g_engine)->setNavigationState(
				OPEN_HOUSE_GAME_TO_MENU);
	}
}

void AndroidPortAdditions::setDisplayDimensions(uint32 width, uint32 height) {
	LOGD("AndroidPortAdditions::setDisplayDimensions: %d %d", width, height);

	mDisplayWidth = width;
	mDisplayHeight = height;

	mDisplayRatio = width / (float) height;
	Drawable::setDisplayRatio(mDisplayRatio);

	mGameToDisplayRatioW = (float) GAME_SCREEN_WIDTH / width;
	mGameToDisplayRatioH = (float) GAME_SCREEN_HEIGHT / height;

// Calculate the touch translation offset based on the height of the black panel, game screen and display
	int16 blackPanelHeight = BLACK_PANEL_END_Y - BLACK_PANEL_START_Y;
// The offset is negative - we fool the system that the touch event was above by this offset
	mTouchTranslationOffset = blackPanelHeight / mGameToDisplayRatioH;
	mTouchTranslationLowerBound = BLACK_PANEL_END_Y / mGameToDisplayRatioH;

	LOGD(
			"AndroidPortAdditions::setDisplayDimensions: mGameToDisplayRatioH %f mTouchTranslationOffset %d mTouchTranslationLowerBound %d",
			mGameToDisplayRatioH, mTouchTranslationOffset,
			mTouchTranslationLowerBound);

// Scaling factor for SW scaler (currently minimal, devices are not fast enough)
	mScalingFactor = 2;

// Calculate scaling factor for LQ shader (according to width)
	mLQShaderScalingFactor = LOW_QUALITY_SHADER_SCALING_FACTOR;//(float)mDisplayWidth / GAME_SCREEN_WIDTH;

	LOGD(
			"AndroidPortAdditions::setDisplayDimensions: mLQShaderScalingFactor %f",
			mLQShaderScalingFactor);

	if (!mScaledOutput) {
		mScaledOutput = new uint16[GAME_SCREEN_WIDTH * GAME_SCREEN_HEIGHT
		* mScalingFactor * mScalingFactor];
	}

	if (!mScalerPlugin) {
		mScalerPlugin = new HQPlugin();
		// Set the pixel format for RGB565
		Graphics::PixelFormat format(2, 5, 6, 5, 0, 11, 5, 0, 0);
		mScalerPlugin->initialize(format);

		if (mScalingFactor > 2) {
			mScalerPlugin->increaseFactor();
		}
	}

	LOGD("AndroidPortAdditions::setDisplayDimensions: mScalingFactor = %d",
			mScalingFactor);

#ifdef MAINTAIN_ASPECT_RATIO
// Calculate the drawing transformation according to aspect ratio
// (needed for games where we adjust the drawing area)
	if (mDisplayRatio < MIN_ASPECT_RATIO) {
		mDrawTransformationWidth = 1.0;
		mDrawTransformationHeight = mDisplayRatio / (float) MIN_ASPECT_RATIO;
	} else if (mDisplayRatio >= MIN_ASPECT_RATIO
			&& mDisplayRatio <= MAX_ASPECT_RATIO) {
		mDrawTransformationWidth = 1.0;
		mDrawTransformationHeight = 1.0;
	} else {

		mDrawTransformationWidth = MIN_ASPECT_RATIO / mDisplayRatio;
		mDrawTransformationHeight = 1.0;
	}

	LOGD(
			"AndroidPortAdditions::setDisplayDimensions: drawing transformation: aspect %f, transformation %f %f",
			mDisplayRatio, mDrawTransformationWidth, mDrawTransformationHeight);

#endif
}

void AndroidPortAdditions::addBitmapResource(String key,
		AndroidBitmap* bitmap) {
	mBitmaps[key] = bitmap;

// Set display dimensions as uninitialized
	bitmap->displayWidth = 0;
	bitmap->displayHeight = 0;
}

AndroidBitmap*
AndroidPortAdditions::getBitmap(String key) {
	return mBitmaps[key];
}

void AndroidPortAdditions::addShaderSource(const char* source, uint32 type) {
	if (type == SHADER_TYPE_HQ_VERTEX) {
		mScalerVertexSource = source;
	} else if (type == SHADER_TYPE_HQ_FRAGMENT) {
		mScalerFragmentSource = source;
	} else if (type == SHADER_TYPE_LQ_VERTEX) {
		mScalerLQVertexSource = source;
	} else if (type == SHADER_TYPE_LQ_FRAGMENT) {
		mScalerLQFragmentSource = source;
	} else if (type == SHADER_TYPE_RECT_HIGHLIGHTING_VERTEX) {
		mRectHighlightingVertexSource = source;
	} else if (type == SHADER_TYPE_RECT_HIGHLIGHTING_FRAGMENT) {
		mRectHighlightingFragmentSource = source;
	}
}

void AndroidPortAdditions::release() {
	LOGD("AndroidPortAdditions: release");

	delete sInstance;
	sInstance = NULL;
}

void AndroidPortAdditions::gameEvent(int32 type) {

	LOGD("AndroidPortAdditions::gameEvent: %d", type);

	switch (type) {
		case GAME_EVENT_SHOULD_TEST_SHADER:

		mShouldTestShader = true;
		break;
		case GAME_EVENT_USE_ULTRA_MODE:

		mShaderScalingMaxResolutionW = SHADER_SCALING_ULTRA_MAX_RESOLUTION_W;
		mShaderScalingMaxResolutionH = SHADER_SCALING_ULTRA_MAX_RESOLUTION_H;
		break;
		case GAME_EVENT_HIDE_NEW_ACTION_BAR:
		mShouldShowNewActionBar = false;
		break;
		case GAME_EVENT_SHOW_NEW_ACTION_BAR:
		mShouldShowNewActionBar = true;
		break;
		case GAME_EVENT_START_GAME_IN_OPEN_HOUSE:
		mStartInOpenHouse = true;
		break;
		case GAME_EVENT_SKIP_CAN_PUZZLE:
		if (getGameType() == GAME_TYPE_T7G) {
			Groovie::GroovieEngine* groovie = (Groovie::GroovieEngine*) g_engine;
			groovie->skipCanPuzzle();
			performSkip(false);
		}
		break;
		case GAME_EVENT_SKIP_BEDSPREAD_PUZZLE:
		if (getGameType() == GAME_TYPE_T7G) {
			Groovie::GroovieEngine* groovie = (Groovie::GroovieEngine*) g_engine;
			groovie->skipBedspreadPuzzle();
			performSkip(false);
		}
		break;
		case GAME_EVENT_SKIP_HEART_PUZZLE:
		if (getGameType() == GAME_TYPE_T7G) {
			Groovie::GroovieEngine* groovie = (Groovie::GroovieEngine*) g_engine;
			groovie->skipHeartPuzzle();
			performSkip(false);
		}
		break;
	}
}

void AndroidPortAdditions::onRenderTimeMeasure(uint64 time) {
	LOGD("AndroidPortAdditions::onRenderTimeMeasure: %lld", time);

	if (!mShouldTestShader) {
		return;
	}

	uint64 currentTime = AndroidPortUtils::getTimeOfDayMillis();

// If this is the first test frame, keep the timestamp
	if (mShaderTestStartTime == 0) {
		mShaderTestStartTime = currentTime;
	}

// Safety check to make sure that the test doesn't take more than a certain time - in order to ensure good user experience
	if (currentTime - mShaderTestStartTime >= RENDER_MEASURE_MAX_TEST_TIME) {
		// Failure in this case
		mShouldTestShader = false;
		mScalingOption == SCALING_OPTION_SHADER ?
		fallbackToLQHardwareScaler() : fallbackToSoftwareScaler();
		return;
	}

// Skip a few frames in the beginning to ignore the time it takes for ScummVM to initialize
	if (mRenderMeasureSkippedFrameCount < RENDER_MEASURE_SKIPPED_FRAMES) {
		++mRenderMeasureSkippedFrameCount;
		return;
	}

// Add to the render time average
	mRenderMeasureAverage += time / (double) RENDER_MEASURE_COUNT_MAX;
	++mRenderMeasureCount;

// Check if we should finish testing
	if (mRenderMeasureCount >= RENDER_MEASURE_COUNT_MAX) {
		mShouldTestShader = false;

		LOGD("AndroidPortAdditions::mRenderMeasureAverage: %f",
				mRenderMeasureAverage);

		// Check the result
		if (mRenderMeasureAverage <= RENDER_MEASURE_THRESHOLD) {
			// The shader has good performance
			JNI::gameEventJNIToJava(GAME_EVENT_SHADER_TEST_SUCCESS);
		} else {
			mScalingOption == SCALING_OPTION_SHADER ?
			fallbackToLQHardwareScaler() : fallbackToSoftwareScaler();

		}
	}
}

void AndroidPortAdditions::fallbackToLQHardwareScaler() {

	LOGD("AndroidPortAdditions::fallbackToLQHardwareScaler: ");

	mScalerShaderProgram = OpenGLESHelper::createProgram(mScalerLQVertexSource,
			mScalerLQFragmentSource);

// Delete the unneeded LQ sources
	delete[] mScalerLQVertexSource;
	delete[] mScalerLQFragmentSource;

	if (mScalerShaderProgram == NULL) {

		LOGD(
				"AndroidPortAdditions::fallbackToLQHardwareScaler: couldn't compile LQ scaler, fallback to SW");

		// Use SW scaler in case of compile error
		fallbackToSoftwareScaler();
	} else {
		LOGD("AndroidPortAdditions::fallbackToLQHardwareScaler: success");

		JNI::gameEventJNIToJava(GAME_EVENT_SCALER_LQ_FALLBACK);
	}
}

void AndroidPortAdditions::fallbackToSoftwareScaler() {
// The shader has bad performance - fallback to software and inform the UI

	if (SUPPORT_SOFTWARE_SCALER)
	mScalingOption = SCALING_OPTION_SOFT;
	else
	mScalingOption = SCALING_OPTION_NONE;

	mScalerShaderProgram = mDefaultShaderProgram;

	OSystem_Android* android = (OSystem_Android*) g_system;
	android->reinitGameTextureSize(GAME_SCREEN_WIDTH, GAME_SCREEN_HEIGHT);

	JNI::gameEventJNIToJava(GAME_EVENT_SHADER_TEST_FAILURE);
}

bool AndroidPortAdditions::checkClick(uint32 x, uint32 y, uint32 buttonX,
		uint32 buttonY, uint32 buttonW, uint32 buttonH) {
	return (x >= buttonX && x < buttonX + buttonW && y >= buttonY
			&& y < buttonY + buttonH);
}

void AndroidPortAdditions::onMouseClick(int x, int y) {
	touchEventSafety(x, y);

// If we are in touch mode and not in chat, recognize an action press
	/*	if (mMouseVisible && mClassicMode && !mGameInChat) {
	 recognizeActionPress(x, y);
	 }*/
}

bool AndroidPortAdditions::checkGameOverlayClicks(int x, int y,
		bool performOverlayAction) {

	uint64 currentTime = AndroidPortUtils::getTimeOfDayMillis();

	if (canSkip()) {
		// Check for "skip" button click
		AndroidBitmap* skip = getBitmap("skip.png");
		if (checkClick(x, y, SKIP_X * mDisplayWidth, SKIP_Y * mDisplayHeight,
						skip->displayWidth, skip->displayHeight)) {

			if (performOverlayAction) {
				bool showSkipAnimation = (getGameType() != GAME_TYPE_T7G);
				performSkip(showSkipAnimation);

				// Mark this flag
				mSkipPressedAtLeastOnce = true;

				// Set the timestamp for last skip press
				mLastTimeSkipPressed = currentTime;
			}

			return true;
		}
	} else {

		if (canShowRevealItems() && !isOpenHouseGrid()) {
			// Check for "reveal items" button click
			AndroidBitmap* revealItems = getBitmap("reveal_items.png");
			if (checkClick(x, y, REVEAL_ITEMS_X * mDisplayWidth,
							REVEAL_ITEMS_Y * mDisplayHeight, revealItems->displayWidth,
							revealItems->displayHeight)) {

				if (performOverlayAction) {
					mShouldPerformRevealItems = true;
				}

				return true;
			}

			// Just a debugging mechanism that can be activated by "reveal items"
#if (defined ANDROID_PORT_DUMP_SCREEN || defined ANDROID_PORT_DUMP_CURSOR)
			mDumpPixelsToFile = true;
#endif

		}
		if (canShowMenuButton()) {
			// Check for menu click
			AndroidBitmap* menu = getBitmap("menu.png");

			// T7G - Menu position might be different in "open house" grid
			float currentMenuX, currentMenuY;
			if (isOpenHouseGrid()) {
				currentMenuX = MAP_X;
				currentMenuY = mLowerButtonPosY;
			} else {
				currentMenuX = MENU_X;
				currentMenuY = MENU_Y;
			}

			if (checkClick(x, y, currentMenuX * mDisplayWidth,
							currentMenuY * mDisplayHeight, menu->displayWidth,
							menu->displayHeight)) {
				if (performOverlayAction) {

					// Check if a minimum delay has passed since pressing skip
					if (currentTime
							- mLastTimeSkipPressed>= MIN_CLICK_DELAY_AFTER_SKIP) {
						// Start the menu
						JNI::gameEventJNIToJava(GAME_EVENT_SHOW_MENU);
					}
				}

				return true;
			}
		}
		if (canReplay() && !isOpenHouseGrid()) {
			// Check for replay click
			AndroidBitmap* replay = getBitmap("replay.png");
			if (checkClick(x, y, REPLAY_X * mDisplayWidth,
							mLowerButtonPosY * mDisplayHeight, replay->displayWidth,
							replay->displayHeight)) {
				if (performOverlayAction) {

					g_engine->replay();
				}

				return true;
			}
		}
		if (canShowMap() && !isOpenHouseGrid()) {
			// Check for map click
			AndroidBitmap* map = getBitmap("map.png");
			if (checkClick(x, y, MAP_X * mDisplayWidth,
							mLowerButtonPosY * mDisplayHeight, map->displayWidth,
							map->displayHeight)) {
				if (performOverlayAction) {

					g_engine->showMap();
				}

				return true;
			}
		}

	}

// Check for chat overlay clicks. Only when in chat mode, not classic mode, and for Simon - when no skip option.
	if (mGameInChat && !mClassicMode && !(isSimonGame() && canSkip())) {
		AndroidBitmap* upArrow = getBitmap("arrow_up.png");
		if (checkClick(x, y, UP_ARROW_X * mDisplayWidth,
						UP_ARROW_Y * mDisplayHeight, upArrow->displayWidth,
						upArrow->displayHeight)) {

			if (performOverlayAction) {
				chatArrowClick(true);
			}

			return true;
		}
		AndroidBitmap* downArrow = getBitmap("arrow_down.png");
		if (checkClick(x, y, DOWN_ARROW_X * mDisplayWidth,
						DOWN_ARROW_Y * mDisplayHeight, downArrow->displayWidth,
						downArrow->displayHeight)) {

			if (performOverlayAction) {
				chatArrowClick(false);
			}

			return true;

		}
		AndroidBitmap* chatButton = getBitmap("talk_btn.png");
		if (checkClick(x, y, CHAT_BUTTON_X * mDisplayWidth,
						CHAT_BUTTON_Y * mDisplayHeight, chatButton->displayWidth,
						chatButton->displayHeight)) {

			if (performOverlayAction) {
				chatButtonClick();
			}

			return true;
		}
	}

	if (mGameInPostcard) {

		int16 gameX = x * mGameToDisplayRatioW;
		int16 gameY = y * mGameToDisplayRatioH;

		// Check for clicks on "save", "load" or "exit"
		uint16 postcardClick = POSTCARD_CLICK_NONE;

		if (gameX >= POSTCARD_SAVE_START_X && gameY >= POSTCARD_SAVE_START_Y
				&& gameX <= POSTCARD_SAVE_END_X && gameY <= POSTCARD_SAVE_END_Y) {
			postcardClick = POSTCARD_CLICK_SAVE;
		} else if (gameX >= POSTCARD_LOAD_START_X
				&& gameY >= POSTCARD_LOAD_START_Y
				&& gameX <= POSTCARD_LOAD_END_X && gameY <= POSTCARD_LOAD_END_Y) {
			postcardClick = POSTCARD_CLICK_LOAD;
		} else if (gameX >= POSTCARD_EXIT_START_X
				&& gameY >= POSTCARD_EXIT_START_Y
				&& gameX <= POSTCARD_EXIT_END_X && gameY <= POSTCARD_EXIT_END_Y) {
			postcardClick = POSTCARD_CLICK_EXIT;
		}

		if (performOverlayAction && postcardClick != POSTCARD_CLICK_NONE) {
			LOGD("AndroidPortAdditions::onTapEvent: postcardClick %d",
					postcardClick);

			// Post a click to the "continue" button to cancel the postcard
			pushClickEvent(POSTCARD_CONTINUE_CLICK_X,
					POSTCARD_CONTINUE_CLICK_Y);

			// Handle the selected option
			JNI::onGameOption(postcardClick);

			return true;
		}

		return false;
	}

	return false;
}

bool AndroidPortAdditions::onTapEvent(int x, int y, bool doubleTap) {
//LOGD("AndroidPortAdditions::onTapEvent %d %d", x, y);

// Dump screen to file on debug mode
#if (defined ANDROID_PORT_DUMP_SCREEN || defined ANDROID_PORT_DUMP_CURSOR)
	if (doubleTap)
	mDumpPixelsToFile = true;
#endif

#ifdef RELEASE_TYPE_DEBUG
//	if (doubleTap)
//		mDebugHideIconsToggle = !mDebugHideIconsToggle;
#endif

	// For IHNM, disable double click
	if (mGameType == GAME_TYPE_IHNM) {
		doubleTap = false;
	}

// Make sure touch events fall within the actual surface
	touchEventSafety(x, y);

	if (checkGameOverlayClicks(x, y)) {
		return true;
	}


	// IHNM maid workaround
	if (getGameType() == GAME_TYPE_IHNM && !mClassicMode && !canShowMenuButton() && !canSkip())
	{
		LOGD("IHNM maid workaround");
		pushClickEvent(0, 0, false);
		return false;
	}

	// Send game event
	if (!mClassicMode
			&& (mMouseVisible || g_engine->canQuitScreenByOneClick())) {
		gameTouchEvent(x, y, 0, 0, doubleTap ? DOUBLE_TAP : TAP);
	}

	return false;
}

bool AndroidPortAdditions::onLongClickEvent(int x, int y) {

// Make sure touch events fall within the actual surface
	touchEventSafety(x, y);

	if (checkGameOverlayClicks(x, y)) {
		return true;
	}

// Send game event
	if (!mClassicMode
			&& (mMouseVisible || g_engine->canQuitScreenByOneClick())) {
		gameTouchEvent(x, y, 0, 0, UP);
	}

	return false;
}

bool AndroidPortAdditions::onFlingEvent(int x1, int y1, int x2, int y2) {
	touchEventSafety(x1, y1);
	touchEventSafety(x2, y2);

// Send game event
	if (!mClassicMode && mMouseVisible) {
		gameTouchEvent(x2, y2, x1, y1, FLING);
	}

	return false;
}

bool AndroidPortAdditions::onScrollEvent(int x, int y) {
//	LOGD("AndroidPortAdditions::onScrollEvent %d %d", x, y);

	touchEventSafety(x, y);

// Send game event
	if (!mClassicMode && mMouseVisible) {
		gameTouchEvent(x, y, 0, 0, DOWN);
	}

	return false;
}

bool AndroidPortAdditions::onDownEvent(int x, int y) {
//	LOGD("AndroidPortAdditions::onDownEvent %d %d", x, y);

	touchEventSafety(x, y);

// Reset walk icon timestamp
	mLastTouchMoveTimestamp = AndroidPortUtils::getTimeOfDayMillis();

// Check if the 'down' was performed on an overlay part - if so, do not pass to game touch behavior
	if (checkGameOverlayClicks(x, y, false)) {
		return false;
	}

// Send game event
	if (!mClassicMode && mMouseVisible) {

		gameTouchEvent(x, y, 0, 0, DOWN);
	}

	return false;
}

bool AndroidPortAdditions::onUpEvent(int x, int y) {
//LOGD("AndroidPortAdditions::onUpEvent %d %d", x, y);

	touchEventSafety(x, y);

// Send game event
	if (!mClassicMode
			&& (mMouseVisible || g_engine->canQuitScreenByOneClick())) {
		gameTouchEvent(x, y, 0, 0, UP);
	}

	return false;
}

/**
 * Using GAME coordinates
 */
void AndroidPortAdditions::pushClickEvent(int32 x, int32 y, bool rightClick) {
	OSystem_Android* android = (OSystem_Android*) g_system;

//	LOGD("AndroidPortAdditions::pushClickEvent: %d %d", x, y);
	android->pushClick(x, y, rightClick);
}

/**
 * Using GAME coordinates
 */
void AndroidPortAdditions::pushScrollEvent(int x, int y) {

	Common::Event e;
	e.type = Common::EVENT_MOUSEMOVE;
	e.mouse.x = x;
	e.mouse.y = y;

	OSystem_Android* android = (OSystem_Android*) g_system;
	android->forceEvent(e);

}

void AndroidPortAdditions::onShowMouse(bool visible) {
//	LOGD("AndroidPortAdditions::onShowMouse: %d", visible);

	if (mMouseVisible && !visible) {
		mMouseJustTurnedInvisible = true;
	}

	mMouseVisible = visible;
}

void AndroidPortAdditions::onGameIdleCounter() {
// Mark a flag if we got an "idle" count while saving
	if (mSaveInProgress) {
		mGameIdleCountWhileSaving = true;
	}
}

void AndroidPortAdditions::onGameNotIdle() {
// Mark a flag if we are not in idle anymore while saving
	if (mSaveInProgress) {
		mGameCancelledIdleWhileSaving = true;
	}
}

bool AndroidPortAdditions::saveGame(int32 slot) {
	LOGD("AndroidPortAdditions::saveGame: %d", slot);

// Generate a simple savename (internal)
	char saveName[20];
	g_engine->generateSaveSlotName(saveName, slot);

// Save the game and check for error
	ErrorCode code = g_engine->saveGameState(slot, saveName).getCode();
	LOGD("AndroidPortAdditions::saveGame: code %d", code);
	return (code == Common::kNoError);
}

bool AndroidPortAdditions::loadGame(int32 slot) {
// Load the game
	return (g_engine->loadGameState(slot).getCode() == Common::kNoError);
}

void AndroidPortAdditions::setSlotToLoad(int32 slot, bool force) {
	mSlotToLoad = slot;
	mForceSaveLoad = force;
}

void AndroidPortAdditions::setSlotToSave(int32 slot, bool force) {
	LOGD("AndroidPortAdditions::setSlotToSave: %d %d", slot, force);

	mSlotToSave = slot;
	mForceSaveLoad = force;
}

void AndroidPortAdditions::setAutoLoadSlot(int32 slot) {
	LOGD("AndroidPortAdditions::setAutoLoadSlot %d", slot);

	mAutoLoadSlot = slot;

	if (slot != -1) {
		mGameStartedInAutoload = true;
		mAutoLoadState = true;
	}
}

bool AndroidPortAdditions::isInAutoloadState() {
// If we are auto-loading, we disable movie fade-out, graphics, etc.

	return mAutoLoadState;
}

void AndroidPortAdditions::onFastFadeInStarted() {
// If we reach the fast fade-in that happens when loading (after we loaded), we end auto-load state.
	if (mAutoLoadSlot == -1) {
		mAutoLoadState = false;
	}
}

void AndroidPortAdditions::initGLESResources() {
// Initialize the GL texture if we didn't yet
	if (mGlTextures == NULL) {
		initGlTextures();
	}

// Init the default program
	mDefaultShaderProgram = OpenGLESHelper::createProgram(
			OpenGLESHelper::DEFAULT_VERTEX_SHADER,
			OpenGLESHelper::DEFAULT_FRAGMENT_SHADER);

// Init the black rect program
	mBlackShaderProgram = OpenGLESHelper::createProgram(
			OpenGLESHelper::BLACK_VERTEX_SHADER,
			OpenGLESHelper::BLACK_FRAGMENT_SHADER);

	if (getGameType() == GAME_TYPE_T7G) {
		// Init the highlighting program
		mRectHighlightingShaderProgram = OpenGLESHelper::createProgram(
				mRectHighlightingVertexSource, mRectHighlightingFragmentSource);
		delete[] mRectHighlightingVertexSource;
		delete[] mRectHighlightingFragmentSource;
	}

	LOGD("AndroidPortAdditions::initGLESResources: support software scaler: %d",
			SUPPORT_SOFTWARE_SCALER);

// Init the scaler program
	if (mScalingOption == SCALING_OPTION_SHADER) {
		//
		// HQ setting behavior
		//

		LOGD("AndroidPortAdditions::initGLESResources: HQ setting behavior");

		mScalerShaderProgram = OpenGLESHelper::createProgram(
				mScalerVertexSource, mScalerFragmentSource);

		if (mScalerShaderProgram == NULL) {
			LOGD(
					"AndroidPortAdditions::initGLESResources: couldn't compile HQ scaler, trying LQ");

			// Load LQ setting in case of compile error
			mScalerShaderProgram = OpenGLESHelper::createProgram(
					mScalerLQVertexSource, mScalerLQFragmentSource);

			// Delete the unneeded LQ sources
			delete[] mScalerLQVertexSource;
			delete[] mScalerLQFragmentSource;

			if (mScalerShaderProgram == NULL) {
				LOGD(
						"AndroidPortAdditions::initGLESResources: couldn't compile LQ scaler, fallback to SW");

				// Use SW scaler in case of compile error
				mScalerShaderProgram = mDefaultShaderProgram;

				if (SUPPORT_SOFTWARE_SCALER)
				mScalingOption = SCALING_OPTION_SOFT;
				else
				mScalingOption = SCALING_OPTION_NONE;

				mShouldTestShader = false;
				JNI::gameEventJNIToJava(GAME_EVENT_SCALER_FALLBACK);
			} else {

				LOGD(
						"AndroidPortAdditions::initGLESResources: fallback to LQ is successful");

				// Inform of LQ fallback
				mScalingOption = SCALING_OPTION_LQ_SHADER;
				mShouldTestShader = false;
				JNI::gameEventJNIToJava(GAME_EVENT_SCALER_LQ_FALLBACK);
			}

		}
	} else if (mScalingOption == SCALING_OPTION_LQ_SHADER) {
		//
		// LQ setting behavior
		//

		LOGD("AndroidPortAdditions::initGLESResources: LQ setting behavior");

		mScalerShaderProgram = OpenGLESHelper::createProgram(
				mScalerLQVertexSource, mScalerLQFragmentSource);

		// Delete the unneeded LQ sources
		delete[] mScalerLQVertexSource;
		delete[] mScalerLQFragmentSource;

		if (mScalerShaderProgram == NULL) {

			LOGD(
					"AndroidPortAdditions::initGLESResources: couldn't compile LQ scaler, fallback to SW");

			// Use SW scaler in case of compile error
			mScalerShaderProgram = mDefaultShaderProgram;

			if (SUPPORT_SOFTWARE_SCALER)
			mScalingOption = SCALING_OPTION_SOFT;
			else
			mScalingOption = SCALING_OPTION_NONE;

			mShouldTestShader = false;
			JNI::gameEventJNIToJava(GAME_EVENT_SCALER_FALLBACK);
		}

	} else {

		//
		// Other setting behavior
		//

		LOGD(
				"AndroidPortAdditions::initGLESResources: SW or original setting behavior");

		mScalerShaderProgram = mDefaultShaderProgram;
	}

// Delete the unneeded HQ sources
	delete[] mScalerVertexSource;
	delete[] mScalerFragmentSource;

	GLCALL(
			glEnableVertexAttribArray(
					mDefaultShaderProgram->mPositionAttributeHandle));
	GLCALL(
			glEnableVertexAttribArray(
					mDefaultShaderProgram->mTexCoordAttributeHandle));
	GLCALL(
			glEnableVertexAttribArray(
					mScalerShaderProgram->mPositionAttributeHandle));
	GLCALL(
			glEnableVertexAttribArray(
					mScalerShaderProgram->mTexCoordAttributeHandle));
	GLCALL(
			glEnableVertexAttribArray(
					mBlackShaderProgram->mPositionAttributeHandle));

	if (getGameType() == GAME_TYPE_T7G) {
		GLCALL(
				glEnableVertexAttribArray(
						mRectHighlightingShaderProgram->mStaticInterpolatorAttributeHandle));
	}

// Init the bitmap that connects to the cursor pixels
	mCursorBitmap = new CursorAndroidBitmap;
	mCursorBitmap->bitmapName = "dynamicCursor";
	mCursorBitmap->sourceContainsAlpha = false;
	mCursorBitmap->glTexture = false;
	mCursorBitmap->drawWithScaling = true;

	mDummyBitmap = new AndroidBitmap;
	mDummyBitmap->sourceContainsAlpha = false;
	mDummyBitmap->glTexture = true;
	mDummyBitmap->drawWithScaling = false;
}

void AndroidPortAdditions::onVideoPlaying(String videoId) {
	LOGD("AndroidPortAdditions::onGameVideoPlaying: %s", videoId.c_str());

// Start playing a subtitle file for the video (if found)
	std::string srtFilename(videoId.c_str());
	srtFilename += ".txt";

	if (mSubtitlePlayer.playSubtitleFile(mGamePath + "/" + srtFilename)) {
		g_engine->fastForward(false);
	}
}

void AndroidPortAdditions::videoSubtitleTick() {
	if (!mSubtitlePlayer.isPlaying() || mSubtitlePlayer.isPaused()) {
		return;
	}

	const SubtitlePlayer::SubtitleState& state = mSubtitlePlayer.updateState();

	switch (state.mState) {
		case SubtitlePlayer::SubtitleState::SHOW:
		LOGD("AndroidPortAdditions::videoSubtitleTick: SHOW: %s",
				state.mCurrentSubtitle.c_str());
		JNI::showSubtitles(state.mCurrentSubtitle.c_str());
		break;
		case SubtitlePlayer::SubtitleState::HIDE:
		LOGD("AndroidPortAdditions::videoSubtitleTick: HIDE");
		JNI::hideSubtitles();
		break;
		default:
		// Do nothing
		break;
	}

}

bool AndroidPortAdditions::isSimonGame() {
	return (getGameType() == GAME_TYPE_SIMON1
			|| getGameType() == GAME_TYPE_SIMON2);
}

uint16 AndroidPortAdditions::getGameType() {
	return CURRENT_GAME_TYPE;
}

void AndroidPortAdditions::onGamePaused() {

	LOGD("AndroidPortAdditions::onGamePaused: ");

	if (mSubtitlePlayer.isPlaying())
	mSubtitlePlayer.pause();
}

void AndroidPortAdditions::onGameResumed() {

	LOGD("AndroidPortAdditions::onGameResumed: ");

	if (mSubtitlePlayer.isPaused())
	mSubtitlePlayer.resume();
}

void AndroidPortAdditions::onDrawTextureToScreen(GLshort x, GLshort y,
		GLshort w, GLshort h) {

//	LOGD("AndroidPortAdditions::onDrawTextureToScreen %d %d %d %d", x, y, w ,h);

// Set the beginning timestamp if it wasn't set yet
	if (mFirstDrawTimestamp == 0) {
		mFirstDrawTimestamp = AndroidPortUtils::getTimeOfDayMillis();
	}

// Draw all animations
	drawAnimations();

// Draw the mouse pointer if needed
	if (mMouseVisible && mClassicMode) {

		drawClassicModeMousePointer();
	}

	// IHNM: cover disk icon in character selection
	if (getGameType() == GAME_TYPE_IHNM) {
		if (((Saga::SagaEngine*) g_engine)->isInCharacterSelection1())
		{
			drawBitmapAsGlTexture(getBitmap("cover_disk.png"), IHNM_COVER_DISK_X,
					IHNM_COVER_DISK_Y, IHNM_COVER_DISK_W, IHNM_COVER_DISK_H);
		}
		else if (((Saga::SagaEngine*) g_engine)->isInCharacterSelection2())
		{
			drawBitmapAsGlTexture(getBitmap("cover_disk2.png"), IHNM_COVER_DISK_X,
					IHNM_COVER_DISK_Y, IHNM_COVER_DISK_W, IHNM_COVER_DISK_H);
		}
	}

// UPDATE: removed for now
// Check if this is the right time to show the label "music enhanced by..."
	/*	uint64 relativeTime = currentTime - mFirstDrawTimestamp;
	 if (relativeTime >= MUSIC_ENHANCED_BY_BEGIN_TIME
	 && relativeTime < MUSIC_ENHANCED_BY_END_TIME
	 && !mSkipPressedAtLeastOnce && !mGameStartedInAutoload) {
	 drawBitmapAsGlTexture(getBitmap("music_enhanced_by.png"),
	 MUSIC_ENHANCED_BY_X, MUSIC_ENHANCED_BY_Y);
	 }*/

// draw the game overlay buttons according to state
	if (canSkip()) {

		drawBitmapAsGlTexture(getBitmap("skip.png"), SKIP_X, SKIP_Y);
	} else {

		// Check if we need to hide the upper icons when revealing items
		float upperIconAlpha = 1.0;
		uint32 revealItemsDuration = REVEAL_ITEMS_FADEIN_DURATION
		+ REVEAL_ITEMS_STAY_DURATION + REVEAL_ITEMS_FADEOUT_DURATION;
		if (AndroidPortUtils::getTimeOfDayMillis() - mLastTimeRevealItemsPressed
				< revealItemsDuration) {
			upperIconAlpha = ICON_ALPHA_WHEN_REVEALING_ITEMS;
		} else if (mUpperTextIsLong) {
			upperIconAlpha = ICON_ALPHA_WHEN_LONG_TEXT;
		}

#ifdef RELEASE_TYPE_DEBUG
		if (mDebugHideIconsToggle) {
			upperIconAlpha = 0.0;
		}
#endif

		// Draw upper icons (for T7G open house grid - different positions)
		if (isOpenHouseGrid()) {
			if (canShowMenuButton()) {
				drawBitmapAsGlTexture(getBitmap("menu.png"), MAP_X,
						mLowerButtonPosY, 0, 0, upperIconAlpha);
			}
		} else {
			if (canShowRevealItems()) {
				drawBitmapAsGlTexture(getBitmap("reveal_items.png"),
						REVEAL_ITEMS_X,
						REVEAL_ITEMS_Y, 0, 0, upperIconAlpha);
			}
			if (canShowMenuButton()) {
				drawBitmapAsGlTexture(getBitmap("menu.png"), MENU_X, MENU_Y, 0,
						0, upperIconAlpha);

				/*	if (getGameType() == GAME_TYPE_FOTAQ) {
				 // TEMP HACK for testing removing of bottom panel
				 drawBitmapAsGlTexture(getBitmap("arrow_down.png"),
				 DOWN_ARROW_X,
				 DOWN_ARROW_Y,
				 ARROW_W, ARROW_H);
				 }*/
			}
			if (canReplay()) {
				drawBitmapAsGlTexture(getBitmap("replay.png"), REPLAY_X,
						mLowerButtonPosY, 0, 0, upperIconAlpha);
			}
			if (canShowMap()) {
				drawBitmapAsGlTexture(getBitmap("map.png"), MAP_X,
						mLowerButtonPosY, 0, 0, upperIconAlpha);
			}
		}

	}

// SIMON 2: Draw bottom panel and verb selection indicator if needed
	if (getGameType() == GAME_TYPE_SIMON2 && !mClassicMode
			&& mShouldShowNewActionBar) {
		drawNewBottomPanelSimon2();
	}

// FOTAQ: Draw bottom panel selection overlays
	if (getGameType() == GAME_TYPE_FOTAQ && !mClassicMode) {
		drawSelectionOverlayFOTAQ();
	}

// Show the chat overlay if needed
	if (mMouseVisible && mGameInChat && !mClassicMode) {
		drawBitmapAsGlTexture(getBitmap("arrow_up.png"), UP_ARROW_X, UP_ARROW_Y,
				ARROW_W,
				ARROW_H);
		drawBitmapAsGlTexture(getBitmap("arrow_down.png"), DOWN_ARROW_X,
				DOWN_ARROW_Y,
				ARROW_W, ARROW_H);
		drawBitmapAsGlTexture(getBitmap("talk_btn.png"), CHAT_BUTTON_X,
				CHAT_BUTTON_Y,
				CHAT_BUTTON_W, CHAT_BUTTON_H);
	}

// Notify the Java layer on the first time we get called here
	if (!mGameDisplayStarted) {
		JNI::onGameDisplayStarted();
		mGameDisplayStarted = true;
	}
}

void AndroidPortAdditions::addWalkFadeoutAnimation(int x, int y) {

	if (getGameType() == GAME_TYPE_IHNM) {
		// IHNM: no action icons
		return;
	}

	DrawablePtr drawable(new Drawable);

	AndroidBitmap* bitmap = getBitmap("walk.png");
	drawable->setBitmap(bitmap);
	drawable->setWidth(SMALL_ACTION_ICON_W);

// Center
	float height = SMALL_ACTION_ICON_W * bitmap->getRatio() * mDisplayRatio;
	float xPos = (x / (float) GAME_SCREEN_WIDTH) - SMALL_ACTION_ICON_W / 2;
	float yPos = (y / (float) GAME_SCREEN_HEIGHT) - height / 2;

// Adjust to bounds
	xPos = MAX(xPos, 0.0f);
	if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM)
	yPos = MAX(yPos, (float) BLACK_PANEL_HEIGHT / GAME_SCREEN_HEIGHT);
	else
	yPos = MAX(yPos, (float) GAME_SCREEN_HEIGHT);

	drawable->setPositionX(xPos);
	drawable->setPositionY(yPos);

// Add an alpha animation to the walk icon
	shared_ptr<AlphaAnimation> fadeInAnimation(new AlphaAnimation);
	fadeInAnimation->setDuration(WALK_ICON_SINGLE_TAP_FADEIN_DURATION);
	fadeInAnimation->setStartAlpha(0);
	fadeInAnimation->setEndAlpha(WALK_ICON_SINGLE_TAP_MAX_ALPHA);
	fadeInAnimation->setInterpolator(
			InterpolatorPtr(new DeccelerateInterpolator));

	shared_ptr<AlphaAnimation> fadeOutAnimation(new AlphaAnimation);
	fadeOutAnimation->setDuration(WALK_ICON_SINGLE_TAP_FADEOUT_DURATION);
	fadeOutAnimation->setStartAlpha(WALK_ICON_SINGLE_TAP_MAX_ALPHA);
	fadeOutAnimation->setEndAlpha(0);
	fadeOutAnimation->setInterpolator(
			InterpolatorPtr(new AccelerateInterpolator));

	shared_ptr<SequenceAnimationComposite> walkAnimation(
			new SequenceAnimationComposite);
	walkAnimation->addAnimation(fadeInAnimation);
	walkAnimation->addAnimation(fadeOutAnimation);

// Add to animation list
	walkAnimation->start(AndroidPortUtils::getTimeOfDayMillis());
	drawable->setAnimation(AnimationPtr(walkAnimation));
	mAnimatedDrawables.push_back(drawable);
}

void AndroidPortAdditions::assistWalkCoordinates(int16& x, int16& y) {

//	LOGD("AndroidPortAdditions::assistWalkCoordinates: old %d %d", x ,y);

	if (!isSimonGame()) {
		return;
	}

// Adjust according to border size to help walking to edges
	if (y < BLACK_PANEL_START_Y) {
		if (x <= WALK_ASSIST_BORDER_SIZE_X) {
			x = 0;
		} else if (x >= GAME_SCREEN_WIDTH - WALK_ASSIST_BORDER_SIZE_X) {
			x = GAME_SCREEN_WIDTH - 1;
		}

		if (y >= BLACK_PANEL_START_Y - WALK_ASSIST_BORDER_SIZE_Y) {
			y = BLACK_PANEL_START_Y - 2;
		}
	}

//LOGD("AndroidPortAdditions::assistWalkCoordinates: new %d %d", x ,y);

}

void AndroidPortAdditions::gameTouchBehavior() {

// Don't run in classic mode
	if (mClassicMode) {
		return;
	}

// Don't run when mouse is invisible
	if (!mMouseVisible && !g_engine->canQuitScreenByOneClick()) {

		// Clear any hotspot leftovers in case we moved to a cutscene (currently enabled for FOTAQ for testing)
		if (mGameType == GAME_TYPE_FOTAQ && mMouseJustTurnedInvisible) {
			// If there was a previous hotspot, fade it out
			clearCurrentHotspotAnimation(HOTSPOT_LEAVE_FADEOUT_DURATION, true,
					false);
			clearCurrentActionIconAnimation(false);

			mCurrentHotspot.clear();

			mMouseJustTurnedInvisible = false;
		}


		return;
	}

	if (mGameType == GAME_TYPE_FOTAQ
			&& ((Queen::QueenEngine*) g_engine)->isDialogPartPlaying()) {
		return;
	}

// Obtain last touch event
	mTouchEventMutex->lock();

	if (mTouchEventQueue.empty()) {
		mTouchEventMutex->unlock();
		return;
	}

	TouchEvent touchEvent = mTouchEventQueue.front();
	mTouchEventQueue.pop();
	int16 gameTouchX = touchEvent.mTouchX;
	int16 gameTouchY = touchEvent.mTouchY;
	TouchState gameTouchState = touchEvent.mTouchState;

	mTouchEventMutex->unlock();

// Flag for whether the current action was performed or not
	bool actionUsed = false;

//	LOGD("AndroidPortAdditions::gameTouchBehavior: %d %d %d",
//	gameTouchX, gameTouchY, gameTouchState);


	switch (gameTouchState) {
		case FLING:
		LOGE(
				"AndroidPortAdditions::gameTouchBehavior: got FLING state here - bug!");
		case NONE:
		// Nothing
		return;
		case DOWN:

		if (g_engine->canQuitScreenByOneClick())
		return;

		if ((isSimonGame() || getGameType() == GAME_TYPE_FOTAQ)
				&& (mGameInPostcard || mGameInChat
						|| gameTouchY >= BLACK_PANEL_END_Y)) {
			// Just a mouse move in non-game area or state
			pushScrollEvent(gameTouchX, gameTouchY);

		} else if (getGameType() == GAME_TYPE_FOTAQ
				&& ((Queen::QueenEngine*) g_engine)->isInPinnacleRoom()) {

			// Just a mouse move in non-game area or state
			pushScrollEvent(gameTouchX, gameTouchY);
		} else if (getGameType() == GAME_TYPE_IHNM

				&& (mGameInChat
						|| (mBottomToolbarAppearing
								&& gameTouchY >= BLACK_PANEL_END_Y))) {
			// Just a mouse move in non-game area or state
			g_engine->forceObjectSelect(0);
			pushScrollEvent(gameTouchX, gameTouchY);

		} else if (isSimonGame() && g_engine->isInScrollingMap()
				&& (gameTouchX < MAP_SCROLL_AREA_W
						|| gameTouchX > GAME_SCREEN_WIDTH - MAP_SCROLL_AREA_W)) {

			// Special behavior for scrolling map: if the touch is in the left or right edges,
			// just push a scroll event to the side, so the map scrolls.
			pushScrollEvent(
					gameTouchX < MAP_SCROLL_AREA_W ? 0 : GAME_SCREEN_WIDTH
					, 0);

			// Reset timestamp for walk action
			mLastTouchMoveTimestamp = AndroidPortUtils::getTimeOfDayMillis();

			// If there was a previous hotspot, add a fadeout animation
			if (!mCurrentHotspot.mDisplayPoint.isOrigin()) {

				clearCurrentHotspotAnimation(HOTSPOT_SWITCH_FADEOUT_DURATION,
						false, false);
				clearCurrentActionIconAnimation(false);
			}

			mCurrentHotspot.clear();
		} else {

			Hotspot newHotspot = mHitAreaHelper.getClosestHotspot(gameTouchX,
					gameTouchY);

			bool isNewHotspot = false;
			if (mCurrentHotspot.mDisplayPoint != newHotspot.mDisplayPoint) {

				isNewHotspot = true;

				// Reset timestamp for walk action
				mLastTouchMoveTimestamp =
				AndroidPortUtils::getTimeOfDayMillis();

				// If there was a previous hotspot, add a fadeout animation
				if (!mCurrentHotspot.mDisplayPoint.isOrigin()) {

					clearCurrentHotspotAnimation(
							HOTSPOT_SWITCH_FADEOUT_DURATION, false, false);
					clearCurrentActionIconAnimation(false);
				}

				if (!newHotspot.mDisplayPoint.isOrigin()) {
					// Hotspot changed - setup the current animation
					LOGD(
							"AndroidPortAdditions::gameTouchBehavior: down: setup hotspot: display %s, cursor %s",
							newHotspot.mDisplayPoint.debugStr(),
							newHotspot.mCursorPoint.debugStr());
					setupCurrentHotspotAnimation(newHotspot);
				}
			}

			// On the game area, check for the nearest hotspot and move the mouse there
			mCurrentHotspot = newHotspot;
			if (!mCurrentHotspot.mCursorPoint.isOrigin()) {

				if (mGameType == GAME_TYPE_IHNM) {
					// For IHNM, we force select using the engine
					g_engine->forceObjectSelect(mCurrentHotspot.mHotspotType);

				}

				if (isNewHotspot && mGameType == GAME_TYPE_FOTAQ
						&& mUseFotaqHotspotWorkaround) {
					((Queen::QueenEngine*) g_engine)->hotspotBugWorkaround(
							true);

					mUseFotaqHotspotWorkaround = false;
				}

				// Move the mouse to the nearest hotspot
				pushScrollEvent(mCurrentHotspot.mCursorPoint.x,
						mCurrentHotspot.mCursorPoint.y);
			} else {
				//	if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ) {
				// If there's no near hotspot, scroll over the touch location
				if (mGameType == GAME_TYPE_IHNM) {
					// For IHNM, we force select using the engine
					g_engine->forceObjectSelect(0);
				}

				pushScrollEvent(gameTouchX, gameTouchY);
				//	}
			}

			if (ADJUST_HOTSPOT_INDICATOR_FOR_FINGER) {
				if (mCurrentHotspot.mHotspotType != HOTSPOT_TYPE_T7G_PUZZLE_1
						&& mCurrentHotspot.mHotspotType
						!= HOTSPOT_TYPE_T7G_PUZZLE_2) {

					// Adjust hotspot for finger position if needed
					adjustHotspotIndicatorForFinger(gameTouchX, gameTouchY);
				}
			}
		}

		break;

		case UP:

		// Reset this flag
		mPreventFingerCoveringHotspot = true;

		if ((isSimonGame() || getGameType() == GAME_TYPE_FOTAQ
						|| getGameType() == GAME_TYPE_IHNM)
				&& (mGameInPostcard || mGameInChat
						|| g_engine->canQuitScreenByOneClick())) {
			LOGD("AndroidPortAdditions::gameTouchBehavior: up click");
			// Just a click in non-game area or state
			pushClickEvent(gameTouchX, gameTouchY);
		} else if (getGameType() == GAME_TYPE_FOTAQ
				&& ((Queen::QueenEngine*) g_engine)->isInPinnacleRoom()) {
			// Just a click in non-game area or state
			pushClickEvent(gameTouchX, gameTouchY);
		} else if ((isSimonGame() || getGameType() == GAME_TYPE_FOTAQ
						|| getGameType() == GAME_TYPE_IHNM)
				&& gameTouchY >= BLACK_PANEL_END_Y) {
			bottomToolbarClickBehavior(gameTouchX, gameTouchY);

			if (getGameType() == GAME_TYPE_FOTAQ)
			mUseFotaqHotspotWorkaround = true;

		} else {

			// Check if we're near a hotspot
			mCurrentHotspot = mHitAreaHelper.getClosestHotspot(gameTouchX,
					gameTouchY);

			if (!mCurrentHotspot.mDisplayPoint.isOrigin()) {

				// Check if an action is selected
				if (mCurrentAction == ACTION_WALK) {
					if (shouldUseLongClickAction()) {

						// For FOTAQ, perform the default action (right click)
						if (getGameType() == GAME_TYPE_FOTAQ) {
							LOGD(
									"AndroidPortAdditions::gameTouchBehavior: FOTAQ: pushing right click to %d %d",
									mCurrentHotspot.mCursorPoint.x,
									mCurrentHotspot.mCursorPoint.y);
							pushClickEvent(mCurrentHotspot.mCursorPoint.x,
									mCurrentHotspot.mCursorPoint.y, true);
						} else {
							if (mGameType == GAME_TYPE_IHNM) {
								// For IHNM, we force click using the engine
								g_engine->forceObjectSelect(
										mCurrentHotspot.mHotspotType);
								g_engine->forceNextObjectClick(true);
							}

							// For other games, If we're on 'walk', walk to the hotspot only if timeout has passes
							pushClickEvent(mCurrentHotspot.mCursorPoint.x,
									mCurrentHotspot.mCursorPoint.y);
						}

						actionUsed = true;
					}
				} else {

					// Otherwise, perform a click on the hotspot to perform the action
					if (mGameType == GAME_TYPE_IHNM) {
						// For IHNM, we force click using the engine
						g_engine->forceObjectSelect(
								mCurrentHotspot.mHotspotType);
						g_engine->forceNextObjectClick(true);

						// IHNM HACK: force autosave after every action
						forceAutosave();
					}
					pushClickEvent(mCurrentHotspot.mCursorPoint.x,
							mCurrentHotspot.mCursorPoint.y);

					actionUsed = true;

				}
			} else {
				// Check if an action is selected
				if (mCurrentAction == ACTION_WALK) {
					// If 'walk' is selected and we're not on a hotspot, walk only if the press is long
					// UPDATE: disabled for now, user can walk using single-tap
					/*if (shouldUseWalkAction()) {
					 // Walk to the current coordinates
					 addWalkFadeoutAnimation(gameTouchX, gameTouchY);
					 assistWalkCoordinates(gameTouchX, gameTouchY);
					 pushClickEvent(gameTouchX, gameTouchY);
					 }*/
				} else {
					// Perform the action on the current coordinates
					// DISABLE for FOTAQ
					if (getGameType() != GAME_TYPE_FOTAQ) {
						pushClickEvent(gameTouchX, gameTouchY);
					}
				}
			}
		}

		if (mGameType == GAME_TYPE_IHNM && !actionUsed) {
			// For IHNM, stop force selection
			g_engine->forceObjectSelect(0);
		}

		// If there was a previous hotspot, fade it out
		clearCurrentHotspotAnimation(HOTSPOT_LEAVE_FADEOUT_DURATION, false,
				actionUsed);
		clearCurrentActionIconAnimation(actionUsed);

		mCurrentHotspot.clear();

		break;

		case TAP:

		// Reset this flag
		mPreventFingerCoveringHotspot = true;

		if ((isSimonGame() || getGameType() == GAME_TYPE_FOTAQ
						|| getGameType() == GAME_TYPE_IHNM)
				&& (mGameInPostcard || mGameInChat
						|| g_engine->canQuitScreenByOneClick())) {
			// Just a click in non-game area or state
			LOGD("AndroidPortAdditions::gameTouchBehavior: tap click");
			pushClickEvent(gameTouchX, gameTouchY);
		} else if (getGameType() == GAME_TYPE_FOTAQ
				&& ((Queen::QueenEngine*) g_engine)->isInPinnacleRoom()) {
			// Just a click in non-game area or state
			pushClickEvent(gameTouchX, gameTouchY);

		} else if ((isSimonGame() || getGameType() == GAME_TYPE_FOTAQ
						|| getGameType() == GAME_TYPE_IHNM)
				&& gameTouchY >= BLACK_PANEL_END_Y) {
			bottomToolbarClickBehavior(gameTouchX, gameTouchY);

			if (getGameType() == GAME_TYPE_FOTAQ)
			mUseFotaqHotspotWorkaround = true;

		} else {
			// Check if we're near a hotspot
			// T7G HACK: On tap action only, put left and right movement on top priority (DISABLED FOR NOW)
			bool performT7GHack = false;//(getGameType() == GAME_TYPE_T7G);
			Hotspot newHotspot = mHitAreaHelper.getClosestHotspot(gameTouchX,
					gameTouchY, performT7GHack);

			if (mCurrentHotspot.mDisplayPoint != newHotspot.mDisplayPoint
					&& !newHotspot.mDisplayPoint.isOrigin()) {

				setupCurrentHotspotAnimation(newHotspot);
			}

			mCurrentHotspot = newHotspot;

			if (!mCurrentHotspot.mDisplayPoint.isOrigin()) {

				// Check if an action is selected
				if ((isSimonGame() || getGameType() == GAME_TYPE_FOTAQ
								|| getGameType() == GAME_TYPE_IHNM)
						&& mCurrentAction == ACTION_WALK
						&& !g_engine->isInScrollingMap()) {
					// If we're on 'walk' (and not in map), walk to the touched point
					addWalkFadeoutAnimation(gameTouchX, gameTouchY);
					assistWalkCoordinates(gameTouchX, gameTouchY);

					// Check if we need outside dragon cave workaround
					if (mCurrentGameArea == OUTSIDE_DRAGON_CAVE_WITH_HOOK) {
						outsideDragonCaveWorkaround(gameTouchX, gameTouchY);
					}

					if (mGameType == GAME_TYPE_IHNM) {
						// For IHNM, we force click using the engine
						g_engine->forceObjectSelect(0);
						g_engine->forceNextObjectClick(true);
					}

					pushClickEvent(gameTouchX, gameTouchY);
				} else {
					LOGD(
							"AndroidPortAdditions::gameTouchBehavior: tap: push click");
					// Otherwise, perform a click on the hotspot to perform the action

					if (mGameType == GAME_TYPE_IHNM) {
						// For IHNM, we force click using the engine
						g_engine->forceObjectSelect(
								mCurrentHotspot.mHotspotType);
						g_engine->forceNextObjectClick(true);

						// IHNM HACK: force autosave after every action
						forceAutosave();
					}

					pushClickEvent(mCurrentHotspot.mCursorPoint.x,
							mCurrentHotspot.mCursorPoint.y);
					actionUsed = true;

				}
			} else if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM) {

				if (mCurrentAction == ACTION_WALK) {
					addWalkFadeoutAnimation(gameTouchX, gameTouchY);
					assistWalkCoordinates(gameTouchX, gameTouchY);
				}

				// Perform action on the touched point
				pushClickEvent(gameTouchX, gameTouchY);
			}
		}

		if (mGameType == GAME_TYPE_IHNM && !actionUsed) {
			// For IHNM, stop force selection
			g_engine->forceObjectSelect(0);
		}

		// If there was a previous hotspot, fade it out
		clearCurrentHotspotAnimation(HOTSPOT_LEAVE_FADEOUT_DURATION, true,
				actionUsed);
		clearCurrentActionIconAnimation(actionUsed);

		mCurrentHotspot.clear();

		break;
		case DOUBLE_TAP:

		// Reset this flag
		mPreventFingerCoveringHotspot = true;

		if (mGameType == GAME_TYPE_IHNM) {
			// For IHNM, stop force selection
			g_engine->forceObjectSelect(0);
		}

		// Force a click regadless of everything
		pushClickEvent(gameTouchX, gameTouchY);

		if (getGameType() == GAME_TYPE_FOTAQ)
		mUseFotaqHotspotWorkaround = true;

		// If there was a previous hotspot, fade it out
		clearCurrentHotspotAnimation(HOTSPOT_LEAVE_FADEOUT_DURATION, false,
				false);
		clearCurrentActionIconAnimation(actionUsed);

		mCurrentHotspot.clear();

		break;
	}

}

void AndroidPortAdditions::adjustHotspotIndicatorForFinger(uint16 touchX,
		uint16 touchY) {

	if (mCurrentHotspotDrawable == NULL || !mPreventFingerCoveringHotspot)
	return;

// Check if the current touch is by a certain distance to the current hotspot
	uint16 hotspotPosX = mCurrentHotspot.mDisplayPoint.x;
	uint16 hotspotPosY = mCurrentHotspot.mDisplayPoint.y;

	if (touchX >= hotspotPosX - ADJUST_HOTSPOT_INDICATOR_FOR_FINGER_MAX_DISTANCE
			&& touchX
			<= hotspotPosX
			+ ADJUST_HOTSPOT_INDICATOR_FOR_FINGER_MAX_DISTANCE
			&& touchY
			>= hotspotPosY
			- ADJUST_HOTSPOT_INDICATOR_FOR_FINGER_MAX_DISTANCE
			&& touchY
			<= hotspotPosY
			+ ADJUST_HOTSPOT_INDICATOR_FOR_FINGER_MAX_DISTANCE) {

		// Adjust hotspot position up or down, depending on the screen position
		if (hotspotPosY > ADJUST_HOTSPOT_INDICATOR_FOR_FINGER_UPPER_AREA) {
			hotspotPosY -= ADJUST_HOTSPOT_INDICATOR_FOR_FINGER_OFFSET;
		} else {
			hotspotPosY += ADJUST_HOTSPOT_INDICATOR_FOR_FINGER_OFFSET;
		}
	}

	mPreventFingerCoveringHotspot = false;

// Set the new position to the drawable
	float drawablePositionX = (hotspotPosX / (float) GAME_SCREEN_WIDTH);
	float drawablePositionY;
	if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM)
	drawablePositionY = ((hotspotPosY + BLACK_PANEL_HEIGHT)
			/ (float) GAME_SCREEN_HEIGHT);
	else
	drawablePositionY = (hotspotPosY / (float) GAME_SCREEN_HEIGHT);

// Adjust to transformation if needed
#ifdef MAINTAIN_ASPECT_RATIO
	drawablePositionX = adjustToDrawTransformationX(drawablePositionX);
	drawablePositionY = adjustToDrawTransformationY(drawablePositionY);
#endif

	mCurrentHotspotDrawable->setPositionX(drawablePositionX);
	mCurrentHotspotDrawable->setPositionY(drawablePositionY);
}

void AndroidPortAdditions::bottomToolbarClickBehavior(int16 gameTouchX,
		int16 gameTouchY) {
// Click on the bottom toolbar
	pushClickEvent(gameTouchX, gameTouchY);
}

void AndroidPortAdditions::setGameTextureInfo(GLuint gameTexture,
		uint16 texWidth, uint16 texHeight) {
	mGameTexture = gameTexture;
	mGameTextureW = texWidth;
	mGameTextureH = texHeight;
}

void AndroidPortAdditions::drawClassicModeMousePointer() {

// Get the mouse position in game coordinates
	const Common::Point &mouse = g_system->getEventManager()->getMousePos();

	if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM) {

		drawBitmapAsGlTexture(getBitmap("cursor.png"),
				mouse.x / (float) GAME_SCREEN_WIDTH,
				mouse.y / (float) GAME_SCREEN_HEIGHT);

	} else if (mGameType == GAME_TYPE_T7G && mCursorBitmap->getPixels() != NULL) {
		// Calculate the correct position for drawing on the screen (According to screen dimensions and transformation if needed)
		float mouseDisplayX = mouse.x / (float) GAME_SCREEN_WIDTH;
		float mouseDisplayY = mouse.y / (float) GAME_SCREEN_HEIGHT;

#ifdef MAINTAIN_ASPECT_RATIO
		// Transform coordinates to our display area
		mouseDisplayX = adjustToDrawTransformationX(mouseDisplayX);
		mouseDisplayY = adjustToDrawTransformationY(mouseDisplayY);
#endif

		// Maintain aspect ratio for the cursor (according to the game's bitmap)
		float cursorW = mCursorBitmap->getWidth() / (float) GAME_SCREEN_WIDTH;
		float cursorH =
		mCursorBitmap->getHeight() / (float) GAME_SCREEN_HEIGHT_ORIGINAL;

		// Center the mouse if needed
		mouseDisplayX = MAX(mouseDisplayX - CLASSIC_CURSOR_W / 2.0, 0.0);
		mouseDisplayY = MAX(mouseDisplayY - cursorH / 2.0, 0.0);

		drawBitmapAsGlTexture(mCursorBitmap, mouseDisplayX, mouseDisplayY,
				cursorW, cursorH);
	}
}

void AndroidPortAdditions::drawAnimations() {
// Get current time
	long time = AndroidPortUtils::getTimeOfDayMillis();

	std::list<DrawablePtr>::iterator it = mAnimatedDrawables.begin();
	while (it != mAnimatedDrawables.end()) {
		DrawablePtr drawable = *it;
		drawable->updateAnimation(time);

		if (drawable->isAnimationFinished()) {
			// If animation was finished, erase element
			mAnimatedDrawables.erase(it++);
		} else {
			// Draw the element
			drawAnimationDrawable(drawable);

			++it;
		}
	}
}

void AndroidPortAdditions::drawAnimationDrawable(const DrawablePtr drawable) {

	float finalPositionX = drawable->getPositionX();
	float finalPositionY = drawable->getPositionY();

	if (drawable->shouldCenter()) {
		finalPositionX -= drawable->getWidth() / 2;
		finalPositionY -= drawable->getHeight() / 2;
	}

	drawBitmapAsGlTexture(drawable->getBitmap(), finalPositionX, finalPositionY,
			drawable->getWidth(), drawable->getHeight(), drawable->getAlpha(),
			drawable->getProgram());
}

void AndroidPortAdditions::drawBitmapAsGlTexture(AndroidBitmap* bitmap, float x,
		float y, float width, float height, float alpha,
		ShaderProgram* customProgram) {
//LOGD("AndroidPortAdditions::drawBitmapAsGlTexture: x %f y %f bitmap: %s w %f h %f alpha %f", x, y, bitmap->bitmapName.c_str(), width, height, alpha);

	uint16 glTextureIndex =
	bitmap->glTexture ? GL_TEXTURE_STATIC : GL_TEXTURE_DYNAMIC;
	GLuint currentTextureWidth =
	bitmap->glTexture ? GL_TEXTURE_WIDTH : GL_DYNAMIC_TEXTURE_WIDTH;
	GLuint currentTextureHeight =
	bitmap->glTexture ? GL_TEXTURE_HEIGHT : GL_DYNAMIC_TEXTURE_HEIGHT;

	GLCALL(glBindTexture(GL_TEXTURE_2D, mGlTextures[glTextureIndex]));

// Use a blending mode which will workaround Android's premultiply alpha behavior
// http://stackoverflow.com/questions/15915337/using-translucent-pngs-as-textures-and-fading-them-opengl-es-2-0
	if (bitmap->sourceContainsAlpha) {
		GLCALL(glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA));
	}

	GLfloat tex_x, tex_y, tex_width, tex_height;

// Texture coordinates should be in GL tex coordinate system(0.0 - 1.0)
	if (bitmap->glTexture) {
		// If the bitmap is already in a GL texture, get the stored coordinates
		tex_x = bitmap->glTextureX / (GLfloat) currentTextureWidth;
		tex_y = bitmap->glTextureY / (GLfloat) currentTextureHeight;
		tex_width = bitmap->getWidth() / (GLfloat) currentTextureWidth;
		tex_height = bitmap->getHeight() / (GLfloat) currentTextureHeight;
	} else {
		//
		// If not, we need to write the bitmap to the dynamic GL texture
		//

		// Clear the dynamic texture to prevent edge artifacts
		GLCALL(
				glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, currentTextureWidth, currentTextureHeight, GL_RGBA, GL_UNSIGNED_SHORT_5_5_5_1, mEmptyClearingBuffer));

		// Write the new bitmap
		GLCALL(
				glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, bitmap->getWidth(), bitmap->getHeight(), GL_RGBA, GL_UNSIGNED_SHORT_5_5_5_1, bitmap->getPixels()));

		tex_x = 0;
		tex_y = 0;
		tex_width = bitmap->getWidth() / (GLfloat) currentTextureWidth;
		tex_height = bitmap->getHeight() / (GLfloat) currentTextureHeight;
	}

// define texCoords (triangle strip) - what part of the texture to crop
// (invert y axis)
	const GLfloat texcoords[] = {tex_x, tex_y, tex_x + tex_width, tex_y, tex_x,
		tex_y + tex_height, tex_x + tex_width, tex_y + tex_height,};

	GLfloat bmpDisplayWidth, bmpDisplayHeight;
	if (width <= 0) {
		// In this scenario, the bitmap dimension are specified in pixels (normal screen coordinates)
		bmpDisplayWidth = bitmap->displayWidth / (GLfloat) mDisplayWidth;
		bmpDisplayHeight = bitmap->displayHeight / (GLfloat) mDisplayHeight;
	} else {
		// Set a custom display width for the bitmap, just for now
		bmpDisplayWidth = width;
		if (height <= 0) {
			bmpDisplayHeight = bmpDisplayWidth * bitmap->getRatio()
			* mDisplayRatio;
		} else {
			bmpDisplayHeight = height;
		}
	}

// Convert x, y, width, height to default GL vertex coordinate system (-1.0 - 1.0)
	x = x * 2 - 1;
	y = y * (-2) + 1;
	bmpDisplayWidth = bmpDisplayWidth * 2;
	bmpDisplayHeight = bmpDisplayHeight * 2;

// define vertices for drawing to the screen
	const GLfloat vertices[] = {x, y, x + bmpDisplayWidth, y, x, y
		- bmpDisplayHeight, x + bmpDisplayWidth, y - bmpDisplayHeight,};

// Choose the shader to use
	bool shouldUseScaler = bitmap->drawWithScaling
	&& (getScalingOption() == SCALING_OPTION_SHADER
			|| getScalingOption() == SCALING_OPTION_LQ_SHADER);
	ShaderProgram* program;

	if (customProgram != NULL) {
		program = customProgram;
	} else if (shouldUseScaler) {
		program = getScalerShaderProgram();
	} else {
		program = getDefaultShaderProgram();
	}

// Use the program
	GLCALL(glUseProgram(program->mProgramHandle));

// Set the texture uniform
	GLCALL(glUniform1i(program->mTextureUniformHandle, 0));

// Set the alpha uniform
	GLCALL(glUniform1f(program->mAlphaFactorUniformHandle, alpha));

// Set extra uniforms for scaler
	if (shouldUseScaler) {
		GLCALL(
				glUniform2f(program->mTextureSizeUniformHandle,
						currentTextureWidth, currentTextureHeight));
		GLCALL(
				glUniform2f(program->mTextureFractUniformHandle,
						1 / (GLfloat )currentTextureWidth,
						1 / (GLfloat )currentTextureHeight));

		GLCALL(
				glUniform2f(program->mInputSizeUniformHandle,
						bitmap->getWidth(), bitmap->getHeight()));
		GLCALL(
				glUniform2f(program->mOutputSizeUniformHandle,
						bmpDisplayWidth * mDisplayWidth / 2,
						bmpDisplayHeight * mDisplayHeight / 2));
	}

// Pass the position attributes
	GLCALL(
			glVertexAttribPointer(program->mPositionAttributeHandle, 2, GL_FLOAT, false, 0, vertices));

// Pass the texture coordinate attributes
	GLCALL(
			glVertexAttribPointer(program->mTexCoordAttributeHandle, 2, GL_FLOAT, false, 0, texcoords));

// Pass the optional static interpolation
	if (program->mStaticInterpolatorAttributeHandle != -1) {
		static GLfloat staticInterpolation[] = {-1.0, -1.0, -1.0, 1.0, 1.0,
			-1.0, 1.0, 1.0};
		GLCALL(
				glVertexAttribPointer(program->mStaticInterpolatorAttributeHandle, 2, GL_FLOAT, false, 0, staticInterpolation));
	}

	GLCALL(glDrawArrays(GL_TRIANGLE_STRIP, 0, 4));

// Revert to default blending mode
	if (bitmap->sourceContainsAlpha) {
		GLCALL(glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA));
	}
}

void AndroidPortAdditions::setupCurrentHotspotAnimation(Hotspot& hotspot) {

// Generate the base hotspot and action drawables
	if (mCurrentHotspotDrawable == NULL) {
		if (mGameType == GAME_TYPE_T7G) {
			shared_ptr<CursorDrawable> cursorDrawable(new CursorDrawable);
			cursorDrawable->setExtraWidthFactor(
					CURSOR_EXTRA_WIDTH_FACTOR_TOUCH_MODE);
			mCurrentHotspotDrawable = cursorDrawable;
		} else {
			mCurrentHotspotDrawable.reset(new Drawable);
		}
	}
	if (mGameType == GAME_TYPE_T7G
			&& hotspot.mHotspotType != HOTSPOT_TYPE_T7G_PUZZLE_1
			&& hotspot.mHotspotType != HOTSPOT_TYPE_T7G_PUZZLE_2
			&& mCurrentHotspotRectDrawable == NULL) {

		// Draw a rect frame if a dimension of the hotspot is bigger than a constant
		if (hotspot.mRect.width() > HOTSPOT_MIN_DIMENSION_FOR_RECT_DRAWING
				|| hotspot.mRect.height()
				> HOTSPOT_MIN_DIMENSION_FOR_RECT_DRAWING)
		mCurrentHotspotRectDrawable.reset(new Drawable);
	}
	if (mCurrentActionDrawable == NULL) {
		mCurrentActionDrawable.reset(new Drawable);
	}

	AndroidBitmap* action;
	if (isSimonGame() || getGameType() == GAME_TYPE_IHNM) {
		action = getActionIcon(mCurrentAction);
	} else if (getGameType() == GAME_TYPE_FOTAQ) {
		// For FOTAQ, use the selected action icon, or the game's default hotspot action when "walk" is selected
		if (mCurrentAction == ACTION_WALK) {
			action = getActionIcon(hotspot.mHotspotType);
		} else {
			action = getActionIcon(mCurrentAction);
		}
	} else if (mGameType == GAME_TYPE_T7G) {
		// TODO: for now disable the purple indicator
		action = NULL;//getBitmap("touch_indicator_back.png");
	}

	AndroidBitmap* touchIndicator;
	if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM)
	touchIndicator = getBitmap("touch_indicator.png");
	else if (mGameType == GAME_TYPE_T7G)
	touchIndicator = mCursorBitmap;

	generateHotspotIndicatorDrawables(touchIndicator, hotspot, action,
			mCurrentHotspotDrawable, mCurrentActionDrawable,
			mCurrentHotspotRectDrawable, NULL);

//
// Setup the hotspot animation
//

// Create a fade in animation
	shared_ptr<AlphaAnimation> hotspotFadeInAnimation(new AlphaAnimation);
	hotspotFadeInAnimation->setDuration(TOUCH_INDICATOR_INITIAL_FADE_DURATION);
	hotspotFadeInAnimation->setStartAlpha(0);
	hotspotFadeInAnimation->setEndAlpha(1);
	hotspotFadeInAnimation->setInterpolator(
			InterpolatorPtr(new DeccelerateInterpolator));

// Create the initial enter animation
	shared_ptr<ParallelAnimation> hotspotEnterAnimation(new ParallelAnimation);
	hotspotEnterAnimation->addAnimation(hotspotFadeInAnimation);

	if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM) {
		// Create the initial scale up animation (in certain games)
		shared_ptr<ScaleAnimation> initialScaleUpAnimation(new ScaleAnimation);
		initialScaleUpAnimation->setStartWidth(TOUCH_INDICATOR_MIN_SCALE_W);
		initialScaleUpAnimation->setEndWidth(TOUCH_INDICATOR_MAX_SCALE_W);
		initialScaleUpAnimation->setDuration(
				TOUCH_INDICATOR_INITIAL_FADE_DURATION);

		hotspotEnterAnimation->addAnimation(initialScaleUpAnimation);
	}

// Create the sequence of enter + loop
	shared_ptr<SequenceAnimationComposite> hotspotAnimation(
			new SequenceAnimationComposite);
	hotspotAnimation->addAnimation(hotspotEnterAnimation);

	if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM) {
		//
		// Create a scale up-down sequence (in certain games)
		//

		// Create the looped scale up animation
		shared_ptr<ScaleAnimation> scaleUpAnimation(new ScaleAnimation);
		scaleUpAnimation->setStartWidth(TOUCH_INDICATOR_MIN_SCALE_W);
		scaleUpAnimation->setEndWidth(TOUCH_INDICATOR_MAX_SCALE_W);
		scaleUpAnimation->setDuration(TOUCH_INDICATOR_SCALE_DURATION);

		// Create a scale down animation
		shared_ptr<ScaleAnimation> scaleDownAnimation(new ScaleAnimation);
		scaleDownAnimation->setStartWidth(TOUCH_INDICATOR_MAX_SCALE_W);
		scaleDownAnimation->setEndWidth(TOUCH_INDICATOR_MIN_SCALE_W);
		scaleDownAnimation->setDuration(TOUCH_INDICATOR_SCALE_DURATION);

		// Create a sequence
		shared_ptr<SequenceAnimationComposite> scaleUpDownAnimation(
				new SequenceAnimationComposite);
		scaleUpDownAnimation->addAnimation(scaleDownAnimation);
		scaleUpDownAnimation->addAnimation(scaleUpAnimation);

		// Create a loop
		shared_ptr<RepeatAnimationWrapper> scaleLoop(
				new RepeatAnimationWrapper(scaleUpDownAnimation, 0));

		hotspotAnimation->addAnimation(scaleLoop);
	} else if (mGameType == GAME_TYPE_T7G) {

		// For T7G, we scale up the indicator once the long click timeout has passed
		class CheckIfLongClickCondition: public Condition {
		public:
			virtual bool evaluate() {
				return AndroidPortAdditions::instance()->shouldUseLongClickAction();
			}
		};

		shared_ptr<WaitForConditionAnimation> longClickConditionAnimation(
				new WaitForConditionAnimation);
		shared_ptr<CheckIfLongClickCondition> longClickCondition(
				new CheckIfLongClickCondition);
		longClickConditionAnimation->setCondition(longClickCondition);

		// Add to the sequence
		hotspotAnimation->addAnimation(longClickConditionAnimation);

		// Create the scale up after the long click condition
		shared_ptr<ScaleAnimation> scaleUpAnimation(new ScaleAnimation);
		// Starting dimension is the current
		scaleUpAnimation->setStartWidth(0);
		scaleUpAnimation->setEndWidth(0);
		scaleUpAnimation->setEndWidthFactor(
				TOUCH_INDICATOR_LONG_CLICK_SCALE_FACTOR);

		scaleUpAnimation->setDuration(TOUCH_INDICATOR_SCALE_DURATION);

		// Add to the sequence
		hotspotAnimation->addAnimation(scaleUpAnimation);

		hotspotAnimation->setFinishOnEnd(false);
	}

// Add the animation
	hotspotAnimation->start(AndroidPortUtils::getTimeOfDayMillis());
	mCurrentHotspotDrawable->setAnimation(hotspotAnimation);

	if (mCurrentHotspotRectDrawable != NULL) {

		// Create a sequence
		shared_ptr<SequenceAnimationComposite> hotspotRectAnimation(
				new SequenceAnimationComposite);

		// Create a fade in animation for the rect
		shared_ptr<AlphaAnimation> rectFadeInAnimation(new AlphaAnimation);
		rectFadeInAnimation->setDuration(TOUCH_INDICATOR_INITIAL_FADE_DURATION);
		rectFadeInAnimation->setStartAlpha(0);
		rectFadeInAnimation->setEndAlpha(HOTSPOT_RECT_END_ALPHA);
		rectFadeInAnimation->setInterpolator(
				InterpolatorPtr(new DeccelerateInterpolator));
		hotspotRectAnimation->addAnimation(rectFadeInAnimation);

		// Add a long click condition to the rect sequence
		class CheckIfLongClickCondition: public Condition {
		public:
			virtual bool evaluate() {
				return AndroidPortAdditions::instance()->shouldUseLongClickAction();
			}
		};
		shared_ptr<WaitForConditionAnimation> longClickConditionAnimation(
				new WaitForConditionAnimation);
		shared_ptr<CheckIfLongClickCondition> longClickCondition(
				new CheckIfLongClickCondition);
		longClickConditionAnimation->setCondition(longClickCondition);
		hotspotRectAnimation->addAnimation(longClickConditionAnimation);

		// Create a fade out animation for the rect
		shared_ptr<AlphaAnimation> rectFadeOutAnimation(new AlphaAnimation);
		// We use the 'scale' duraiton because this fade out happens with the icon scaling simultaneously
		rectFadeOutAnimation->setDuration(TOUCH_INDICATOR_SCALE_DURATION);
		rectFadeOutAnimation->setStartAlpha(HOTSPOT_RECT_END_ALPHA);
		rectFadeOutAnimation->setEndAlpha(0);
		rectFadeOutAnimation->setInterpolator(
				InterpolatorPtr(new DeccelerateInterpolator));
		hotspotRectAnimation->addAnimation(rectFadeOutAnimation);

		hotspotRectAnimation->start(AndroidPortUtils::getTimeOfDayMillis());
		mCurrentHotspotRectDrawable->setAnimation(hotspotRectAnimation);
	}

	if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM) {
		// Push the hotspot drawable before the action
		mAnimatedDrawables.push_back(mCurrentHotspotDrawable);
	}

//
// Setup the action icon animation
//

	AnimationPtr actionIconAnimation;

// Create a fade in animation
	shared_ptr<AlphaAnimation> actionFadeInAnimation(new AlphaAnimation);
	actionFadeInAnimation->setDuration(ACTION_ENTER_FADEIN_DURATION);
	actionFadeInAnimation->setStartAlpha(0);
	actionFadeInAnimation->setEndAlpha(ACTION_ALPHA);
	actionFadeInAnimation->setInterpolator(
			InterpolatorPtr(new DeccelerateInterpolator));
	actionFadeInAnimation->start(AndroidPortUtils::getTimeOfDayMillis());

	if (mCurrentAction == ACTION_WALK) {

		mCurrentActionDrawable->setAlpha(0);

		// If using WALK, create a conditional wait animation for using before the fade-in (using a local class)
		class CheckIfWalkCondition: public Condition {
		public:
			virtual bool evaluate() {
				return AndroidPortAdditions::instance()->shouldUseLongClickAction();
			}
		};

		shared_ptr<WaitForConditionAnimation> walkConditionAnimation(
				new WaitForConditionAnimation);
		shared_ptr<CheckIfWalkCondition> walkCondition(
				new CheckIfWalkCondition);
		walkConditionAnimation->setCondition(walkCondition);

		// Create the sequence of wait + fade in
		shared_ptr<SequenceAnimationComposite> sequence(
				new SequenceAnimationComposite);
		sequence->addAnimation(walkConditionAnimation);
		sequence->addAnimation(actionFadeInAnimation);
		actionIconAnimation = sequence;
	} else {
		actionIconAnimation = actionFadeInAnimation;
	}

// Add the animation
	actionIconAnimation->setFinishOnEnd(false);
	actionIconAnimation->start(AndroidPortUtils::getTimeOfDayMillis());
	mCurrentActionDrawable->setAnimation(actionIconAnimation);

	if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ) {
		mAnimatedDrawables.push_back(mCurrentActionDrawable);
	}

	if (getGameType() == GAME_TYPE_IHNM) {

		if (!(((Saga::SagaEngine*) g_engine)->isInCharacterSelection1() || ((Saga::SagaEngine*) g_engine)->isInCharacterSelection2())) {
		// IHNM: no action icons
		//	mAnimatedDrawables.push_back(mCurrentActionDrawable);
	}
	}


	if (mGameType == GAME_TYPE_T7G) {
		if (mCurrentHotspotRectDrawable != NULL) {
			mAnimatedDrawables.push_back(mCurrentHotspotRectDrawable);
		}

		// Push the hotspot drawable after the highlighting rect
		mAnimatedDrawables.push_back(mCurrentHotspotDrawable);
	}
}

void AndroidPortAdditions::clearCurrentHotspotAnimation(long fadeoutDuration,
		bool completeFadeIn, bool actionUsed) {

// Make sure we have a current hotspot reference
	if (mCurrentHotspotDrawable == NULL) {
		return;
	}

	if (mGameType == GAME_TYPE_T7G) {
		// Create a snapshot of the drawable's bitmap (because the cursor is dynamic)
		mCurrentHotspotDrawable->takeBitmapSnapshot();
	}

// Set an animation that will erase the drawable at the end
// For Simon games: including scale down
// For T7G - blink if action was used
	shared_ptr<Animation> disappearFadeAnimation;

	if (mGameType == GAME_TYPE_T7G && actionUsed) {

		// No need for that, since we blink anyway
		completeFadeIn = false;

		//
		// Blink, then fade out
		//

		// Create the initial fade in animation
		shared_ptr<AlphaAnimation> initialFadeInAnimation(new AlphaAnimation());
		initialFadeInAnimation->setDuration(HOTSPOT_BLINK_FADE_DURATION);
		initialFadeInAnimation->setStartAlpha(
				mCurrentHotspotDrawable->getAlpha());
		initialFadeInAnimation->setEndAlpha(1);

		// Create the loop fade in animation
		shared_ptr<AlphaAnimation> loopFadeInAnimation(new AlphaAnimation());
		loopFadeInAnimation->setDuration(HOTSPOT_BLINK_FADE_DURATION);
		loopFadeInAnimation->setStartAlpha(0);
		loopFadeInAnimation->setEndAlpha(1);

		// Create a fade out animation
		shared_ptr<AlphaAnimation> fadeOutAnimation(new AlphaAnimation());
		fadeOutAnimation->setDuration(HOTSPOT_BLINK_FADE_DURATION);
		fadeOutAnimation->setStartAlpha(1);
		fadeOutAnimation->setEndAlpha(0);

		// Create a sequence for fade in \ fade out
		shared_ptr<SequenceAnimationComposite> fadeSequenceAnimation(
				new SequenceAnimationComposite());
		fadeSequenceAnimation->addAnimation(loopFadeInAnimation);
		fadeSequenceAnimation->addAnimation(fadeOutAnimation);

		// Create a repeatable animation
		// We repeat times - 1 because the first "repeat" is done separately, to consider the starting alpha value
		shared_ptr<RepeatAnimationWrapper> repeatAnimation(
				new RepeatAnimationWrapper(fadeSequenceAnimation,
						HOTSPOT_BLINK_REPEAT_COUNT - 1));

		// Create a sequence of the initial fade in, and the loop
		shared_ptr<SequenceAnimationComposite> hotspotBlinkAnimation(
				new SequenceAnimationComposite());
		hotspotBlinkAnimation->addAnimation(initialFadeInAnimation);
		hotspotBlinkAnimation->addAnimation(fadeOutAnimation);
		hotspotBlinkAnimation->addAnimation(repeatAnimation);

		// Set the blink animation as our "fade out"
		disappearFadeAnimation = hotspotBlinkAnimation;
	} else {
		//
		// Simple fade out
		//
		shared_ptr<AlphaAnimation> fadeOutAnimation(new AlphaAnimation());
		fadeOutAnimation->setDuration(fadeoutDuration);
		fadeOutAnimation->setStartAlpha(mCurrentHotspotDrawable->getAlpha());
		fadeOutAnimation->setEndAlpha(0);
		fadeOutAnimation->setInterpolator(
				InterpolatorPtr(new DeccelerateInterpolator));

		disappearFadeAnimation = fadeOutAnimation;
	}

	shared_ptr<ParallelAnimation> hotspotDisappearAnimation(
			new ParallelAnimation);
	hotspotDisappearAnimation->addAnimation(disappearFadeAnimation);

	if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ | getGameType() == GAME_TYPE_IHNM) {

		// Create a scale down animation
		shared_ptr<ScaleAnimation> scaleDownAnimation(new ScaleAnimation);
		scaleDownAnimation->setStartWidth(mCurrentHotspotDrawable->getWidth());
		scaleDownAnimation->setEndWidth(TOUCH_INDICATOR_MIN_SCALE_W);
		scaleDownAnimation->setDuration(fadeoutDuration);
		scaleDownAnimation->setInterpolator(
				InterpolatorPtr(new DeccelerateInterpolator));
		hotspotDisappearAnimation->addAnimation(scaleDownAnimation);
	}

	AnimationPtr hotspotClearingAnimation;

// Check whether to complete a fade-in before clearing the hotspot
	if (completeFadeIn) {
		// Create a fade in animation
		shared_ptr<AlphaAnimation> hotspotFadeInAnimation(new AlphaAnimation);
		hotspotFadeInAnimation->setDuration(
				TOUCH_INDICATOR_INITIAL_FADE_DURATION);
		hotspotFadeInAnimation->setStartAlpha(
				mCurrentHotspotDrawable->getAlpha());
		hotspotFadeInAnimation->setEndAlpha(1);
		hotspotFadeInAnimation->setInterpolator(
				InterpolatorPtr(new DeccelerateInterpolator));

		shared_ptr<SequenceAnimationComposite> fadeInOutSequence(
				new SequenceAnimationComposite);
		fadeInOutSequence->addAnimation(hotspotFadeInAnimation);
		fadeInOutSequence->addAnimation(hotspotDisappearAnimation);

		hotspotClearingAnimation = fadeInOutSequence;
	} else {
		hotspotClearingAnimation = hotspotDisappearAnimation;
	}

	hotspotClearingAnimation->start(AndroidPortUtils::getTimeOfDayMillis());
	mCurrentHotspotDrawable->setAnimation(hotspotClearingAnimation);

	if (mCurrentHotspotRectDrawable != NULL) {

		//
		// Create a similar animation for the rect
		//
		shared_ptr<AlphaAnimation> alphaAnimation(new AlphaAnimation);
		alphaAnimation->setDuration(fadeoutDuration);
		alphaAnimation->setStartAlpha(mCurrentHotspotRectDrawable->getAlpha());
		alphaAnimation->setEndAlpha(0);
		alphaAnimation->setInterpolator(
				InterpolatorPtr(new DeccelerateInterpolator));

		shared_ptr<ParallelAnimation> hotspotFadeOutAnimation(
				new ParallelAnimation);
		hotspotFadeOutAnimation->addAnimation(alphaAnimation);

		AnimationPtr hotspotClearingAnimation;

		// Check whether to complete a fade-in before clearing the hotspot
		if (completeFadeIn) {
			// Create a fade in animation
			shared_ptr<AlphaAnimation> hotspotFadeInAnimation(
					new AlphaAnimation);
			hotspotFadeInAnimation->setDuration(
					TOUCH_INDICATOR_INITIAL_FADE_DURATION);
			hotspotFadeInAnimation->setStartAlpha(
					mCurrentHotspotDrawable->getAlpha());
			hotspotFadeInAnimation->setEndAlpha(HOTSPOT_RECT_END_ALPHA);
			hotspotFadeInAnimation->setInterpolator(
					InterpolatorPtr(new DeccelerateInterpolator));

			shared_ptr<SequenceAnimationComposite> fadeInOutSequence(
					new SequenceAnimationComposite);
			fadeInOutSequence->addAnimation(hotspotFadeInAnimation);
			fadeInOutSequence->addAnimation(hotspotFadeOutAnimation);

			hotspotClearingAnimation = fadeInOutSequence;
		} else {
			hotspotClearingAnimation = hotspotFadeOutAnimation;
		}

		hotspotClearingAnimation->start(AndroidPortUtils::getTimeOfDayMillis());
		mCurrentHotspotRectDrawable->setAnimation(hotspotClearingAnimation);
	}

// Clear the reference
	mCurrentHotspotDrawable = NULL;
	mCurrentHotspotRectDrawable = NULL;

}

void AndroidPortAdditions::generateHotspotIndicatorDrawables(
		AndroidBitmap* bitmap, Hotspot& hotspot, AndroidBitmap* action,
		DrawablePtr hotspotDrawable, DrawablePtr actionDrawable,
		DrawablePtr hotspotRectDrawable, float alpha) {

	int16 x = hotspot.mDisplayPoint.x;
	int16 y = hotspot.mDisplayPoint.y;

	if (hotspotDrawable != NULL) {
// Draw an indicator on the center of the target
// (calculate the starting corrdinates for drawing so that the bitmap is centered on the hotspot)
		if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM)
		hotspotDrawable->setWidth(TOUCH_INDICATOR_W);

		// getWidth might return a dynamic value, so we use it
		float indicatorHeight = hotspotDrawable->getWidth() * bitmap->getRatio()
		* mDisplayRatio;
		float hotspotPositionX = (x / (float) GAME_SCREEN_WIDTH);
		float hotspotPositionY;
		if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM)
		hotspotPositionY = ((y + (mClassicMode ? 0 : BLACK_PANEL_HEIGHT))
				/ (float) GAME_SCREEN_HEIGHT);
		else
		hotspotPositionY = (y / (float) GAME_SCREEN_HEIGHT);

		// Adjust to transformation if needed
#ifdef MAINTAIN_ASPECT_RATIO
		hotspotPositionX = adjustToDrawTransformationX(hotspotPositionX);
		hotspotPositionY = adjustToDrawTransformationY(hotspotPositionY);
#endif

		hotspotDrawable->setPositionX(hotspotPositionX);
		hotspotDrawable->setPositionY(hotspotPositionY);
		hotspotDrawable->setShouldCenter(true);
		hotspotDrawable->setAlpha(alpha);
		hotspotDrawable->setBitmap(bitmap);

		if (action != NULL) {
			// Draw action icon

			float actionY, actionWidth;
			if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM) {
				// In Simon, Y position is above the hotspot
				float actionHeight = SMALL_ACTION_ICON_W * action->getRatio()
				* mDisplayRatio;
				actionY = hotspotPositionY - 0.01 - actionHeight;
				if (actionY
						< (BLACK_PANEL_HEIGHT / (float) GAME_SCREEN_HEIGHT)) {
					// If we're on the upper part, switch Y value to be below the indicator
					actionY = hotspotPositionY + indicatorHeight + 0.06;
				}

				actionWidth = SMALL_ACTION_ICON_W;
			} else if (mGameType == GAME_TYPE_T7G) {
				// In T7G, Y position is centered on the hotspot
				float actionHeight = TOUCH_INDICATOR_BACK_W * action->getRatio()
				* mDisplayRatio;
				actionY = (y / (float) GAME_SCREEN_HEIGHT) - actionHeight / 2;

				actionWidth = TOUCH_INDICATOR_BACK_W;
			}

			float actionX = (x / (float) GAME_SCREEN_WIDTH) - actionWidth / 2;

#ifdef MAINTAIN_ASPECT_RATIO
			actionX = adjustToDrawTransformationX(actionX);
			actionY = adjustToDrawTransformationY(actionY);
#endif

			// X position is in the center
			actionDrawable->setPositionX(actionX);
			actionDrawable->setPositionY(actionY);
			actionDrawable->setWidth(actionWidth);
			actionDrawable->setBitmap(action);
		}
	}

	if (hotspotRectDrawable != NULL) {

		float rectPositionX = hotspot.mRect.left / (float) GAME_SCREEN_WIDTH;
		float rectPositionY = hotspot.mRect.top / (float) GAME_SCREEN_HEIGHT;
		float rectWidth = hotspot.mRect.width() / (float) GAME_SCREEN_WIDTH;
		float rectHeight = hotspot.mRect.height() / (float) GAME_SCREEN_HEIGHT;

#ifdef MAINTAIN_ASPECT_RATIO
		rectPositionX = adjustToDrawTransformationX(rectPositionX);
		rectPositionY = adjustToDrawTransformationY(rectPositionY);
		rectWidth *= mDrawTransformationWidth;
		rectHeight *= mDrawTransformationHeight;
#endif

		hotspotRectDrawable->setPositionX(rectPositionX);
		hotspotRectDrawable->setPositionY(rectPositionY);
		hotspotRectDrawable->setWidth(rectWidth);
		hotspotRectDrawable->setHeight(rectHeight);
		hotspotRectDrawable->setAlpha(alpha);
		hotspotRectDrawable->setBitmap(mDummyBitmap);
		hotspotRectDrawable->setProgram(mRectHighlightingShaderProgram);
	}

}

void AndroidPortAdditions::drawNewBottomPanelSimon2() {
// Check if we need to draw the bottom panel
	if (mBottomToolbarAppearing) {
		drawBitmapAsGlTexture(getBitmap("bottom_panel_simon2.png"),
				SIMON2_BOTTOM_PANEL_X,
				SIMON2_BOTTOM_PANEL_Y,
				SIMON2_BOTTOM_PANEL_W,
				SIMON2_BOTTOM_PANEL_H);
	}

// Determine the coordinates for drawing the indicator
	float x, y;
	switch (getCurrentAction()) {
		case ACTION_MOVE:
		x = SIMON2_VERB_INDICATOR_COL1_X;
		y = SIMON2_VERB_INDICATOR_ROW1_Y;
		break;
		case ACTION_LOOK:
		x = SIMON2_VERB_INDICATOR_COL2_X;
		y = SIMON2_VERB_INDICATOR_ROW1_Y;
		break;
		case ACTION_TALK:
		x = SIMON2_VERB_INDICATOR_COL3_X;
		y = SIMON2_VERB_INDICATOR_ROW1_Y;
		break;
		case ACTION_WEAR:
		x = SIMON2_VERB_INDICATOR_COL4_X;
		y = SIMON2_VERB_INDICATOR_ROW1_Y;
		break;
		case ACTION_OPEN:
		x = SIMON2_VERB_INDICATOR_COL1_X;
		y = SIMON2_VERB_INDICATOR_ROW2_Y;
		break;
		case ACTION_PICK:
		x = SIMON2_VERB_INDICATOR_COL2_X;
		y = SIMON2_VERB_INDICATOR_ROW2_Y;
		break;
		case ACTION_USE:
		x = SIMON2_VERB_INDICATOR_COL3_X;
		y = SIMON2_VERB_INDICATOR_ROW2_Y;
		break;
		case ACTION_GIVE:
		x = SIMON2_VERB_INDICATOR_COL4_X;
		y = SIMON2_VERB_INDICATOR_ROW2_Y;
		break;
		default:
		// Do nothing
		return;
	}

// Draw the indicator
	drawBitmapAsGlTexture(getBitmap("verb_selected.png"), x, y,
			SIMON2_VERB_INDICATOR_W,
			SIMON2_VERB_INDICATOR_H);
}

void AndroidPortAdditions::drawSelectionOverlayFOTAQ() {
// Check if the bottom panel is showing
	if (!mBottomToolbarAppearing) {
		return;
	}

// Determine the coordinates for drawing the indicator
	float x, y = FOTAQ_SELECTION_OVERLAY_Y;
	switch (mCurrentAction) {
		case ACTION_OPEN:
		x = FOTAQ_SELECTION_OVERLAY_X_1;
		break;
		case ACTION_CLOSE:
		x = FOTAQ_SELECTION_OVERLAY_X_2;
		break;
		case ACTION_MOVE:
		x = FOTAQ_SELECTION_OVERLAY_X_3;
		break;
		case ACTION_GIVE:
		x = FOTAQ_SELECTION_OVERLAY_X_4;
		break;
		case ACTION_LOOK:
		x = FOTAQ_SELECTION_OVERLAY_X_5;
		break;
		case ACTION_PICK:
		x = FOTAQ_SELECTION_OVERLAY_X_6;
		break;
		case ACTION_TALK:
		x = FOTAQ_SELECTION_OVERLAY_X_7;
		break;
		case ACTION_USE:
		x = FOTAQ_SELECTION_OVERLAY_X_8;
		break;
		default:
		// Do nothing
		return;
	}

// Draw the indicator
	drawBitmapAsGlTexture(getBitmap("selection_overlay.png"), x, y,
			FOTAQ_SELECTION_OVERLAY_W,
			FOTAQ_SELECTION_OVERLAY_H);
}

uint16*
AndroidPortAdditions::scale(const uint8 *srcPtr, uint16 x, uint16 y, uint16 w,
		uint16 h) {
	uint32 srcPitch = GAME_SCREEN_WIDTH * 2;
	uint32 dstPitch = w * mScalingFactor * 2;

	mScalerPlugin->scale(srcPtr, srcPitch, (uint8*) mScaledOutput, dstPitch, w,
			h, x, y);

	return mScaledOutput;
}

void AndroidPortAdditions::setScalingOption(uint16 option) {
	LOGD("AndroidPortAdditions::setScalingOption %d", option);

	mScalingOption = option;
}

void AndroidPortAdditions::initGlTextures() {
	LOGD("AndroidPortAdditions::initGlTextures");

// We need 2 textures - one for static images, and one for dynamic bitmap drawing
	mGlTextures = new GLuint[2];
	GLCALL(glGenTextures(2, mGlTextures));

// Initialize the texture for static images
	GLCALL(glBindTexture(GL_TEXTURE_2D, mGlTextures[GL_TEXTURE_STATIC]));
	GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR));
	GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE));
	GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE));
	byte* combinedGlTexturePixels = new byte[GL_TEXTURE_WIDTH
	* GL_TEXTURE_HEIGHT * 4];
	GLCALL(
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, GL_TEXTURE_WIDTH, GL_TEXTURE_HEIGHT, 0, GL_RGBA, GL_UNSIGNED_BYTE, combinedGlTexturePixels));
	delete[] combinedGlTexturePixels;

	initBitmapInGlTextures(getBitmap("skip.png"), SKIP_W);
	initBitmapInGlTextures(getBitmap("reveal_items.png"), REVEAL_ITEMS_W);
	initBitmapInGlTextures(getBitmap("close.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("consume.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("give.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("look.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("move.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("open.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("pick.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("remove.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("talk.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("talk_btn.png"), CHAT_BUTTON_W, true);
	initBitmapInGlTextures(getBitmap("menu.png"), MENU_W);
	initBitmapInGlTextures(getBitmap("use.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("walk.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("wear.png"), SMALL_ACTION_ICON_W);
	initBitmapInGlTextures(getBitmap("cursor.png"), CLASSIC_CURSOR_W);
	initBitmapInGlTextures(getBitmap("touch_indicator.png"), TOUCH_INDICATOR_W);
	initBitmapInGlTextures(getBitmap("arrow_up.png"), ARROW_W, true);
	initBitmapInGlTextures(getBitmap("arrow_down.png"), ARROW_W, true);

	if (getGameType() == GAME_TYPE_SIMON2) {
		initBitmapInGlTextures(getBitmap("verb_selected.png"),
				SIMON2_VERB_INDICATOR_W, true);
		initBitmapInGlTextures(getBitmap("bottom_panel_simon2.png"),
				SIMON2_BOTTOM_PANEL_W, true);
	}

	if (getGameType() == GAME_TYPE_FOTAQ) {
		initBitmapInGlTextures(getBitmap("selection_overlay.png"),
				FOTAQ_SELECTION_OVERLAY_W, true);
	}

	if (getGameType() == GAME_TYPE_IHNM) {
		initBitmapInGlTextures(getBitmap("cover_disk.png"), 0, false);
		initBitmapInGlTextures(getBitmap("cover_disk2.png"), 0, true);
	}

	if (getGameType() == GAME_TYPE_T7G) {
		initBitmapInGlTextures(getBitmap("square.png"), 0);
		initBitmapInGlTextures(getBitmap("replay.png"), LOWER_BUTTON_W);
		initBitmapInGlTextures(getBitmap("map.png"), LOWER_BUTTON_W);
		initBitmapInGlTextures(getBitmap("touch_indicator_2.png"),
				TOUCH_INDICATOR_W);
		initBitmapInGlTextures(getBitmap("touch_indicator_3.png"),
				TOUCH_INDICATOR_W);

		// Init the Y position for lower buttons
		float lowerButtonHeight = getBitmap("replay.png")->displayHeight
		/ (float) mDisplayHeight;
		mLowerButtonPosY = 1.0 - lowerButtonHeight;
	}

// Initialize the texture for dynamic bitmap drawing
	GLCALL(glBindTexture(GL_TEXTURE_2D, mGlTextures[GL_TEXTURE_DYNAMIC]));
	GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST));
	GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE));
	GLCALL(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE));
	uint16* dynamicTexturePixels = new uint16[GL_DYNAMIC_TEXTURE_WIDTH
	* GL_DYNAMIC_TEXTURE_HEIGHT];
	GLCALL(
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, GL_DYNAMIC_TEXTURE_WIDTH, GL_DYNAMIC_TEXTURE_WIDTH, 0, GL_RGBA, GL_UNSIGNED_SHORT_5_5_5_1, dynamicTexturePixels));
	delete[] dynamicTexturePixels;
}

void AndroidPortAdditions::initBitmapInGlTextures(AndroidBitmap* bitmap,
		float width, bool sourceContainsAlpha) {
// This generates the right coordinates of the bitmap inside the GL texture
	mGlTextureHelper.allocateBitmapInGlTexture(bitmap);

	GLCALL(
			glTexSubImage2D(GL_TEXTURE_2D, 0, bitmap->glTextureX, bitmap->glTextureY, bitmap->getWidth(), bitmap->getHeight(), GL_RGBA, GL_UNSIGNED_BYTE, bitmap->getPixels()));

// Calculate ratio
	bitmap->setRatio((float) bitmap->getHeight() / (float) bitmap->getWidth());

// Calculate sizes
	bitmap->displayWidth = mDisplayWidth * width;
	bitmap->displayHeight = bitmap->displayWidth * bitmap->getRatio();// Get the height by multiplying the image aspect ratio

// Save this flag per bitmap
	bitmap->sourceContainsAlpha = sourceContainsAlpha;
	bitmap->glTexture = true;
	bitmap->drawWithScaling = false;

// We don't need the bitmap in memory anymore
	delete[] (byte*) bitmap->getPixels();
	bitmap->setPixels(NULL);
}

void AndroidPortAdditions::performSkip(bool showAnimation) {
	LOGD("AndroidPortAdditions::performSkip()");

	mSubtitlePlayer.reset();
	JNI::hideSubtitles();

// For T7G, if we're in the long intro, just load the first game save
// TODO: disable for now
	/*if (mGameType == GAME_TYPE_T7G && !mAfterGameIntro) {
	 LOGD("AndroidPortAdditions::performSkip: T7G: skipping intro");
	 JNI::gameEventJNIToJava(GAME_EVENT_SKIP_INTRO);
	 return;
	 }*/

	if (showAnimation) {
		addBigActionFadeAnimation(getBitmap("skip.png"));
	}

	OSystem_Android* android = (OSystem_Android*) g_system;

	Common::Event e1, e2, e3, e4;

	if (mGameType == GAME_TYPE_SIMON1 || mGameType == GAME_TYPE_SIMON2
			|| mGameType == GAME_TYPE_IHNM) {
// Push the "esc" key event to the system
		e1.kbd.keycode = Common::KEYCODE_ESCAPE;
		e1.kbd.ascii = Common::ASCII_ESCAPE;
		e1.type = Common::EVENT_KEYDOWN;
		android->forceEvent(e1);

		e2.kbd.keycode = Common::KEYCODE_ESCAPE;
		e2.kbd.ascii = Common::ASCII_ESCAPE;
		e2.type = Common::EVENT_KEYUP;
		android->forceEvent(e2);

		// Push a right click event to the system
		e3.type = Common::EVENT_RBUTTONDOWN;
		android->forceEvent(e3);

		e4.type = Common::EVENT_RBUTTONUP;
		android->forceEvent(e4);
	} else if (mGameType == GAME_TYPE_FOTAQ) {

		KeyCode keyCode;
		uint16 ascii;
		if (g_engine->isDialogPartPlaying() && !mGameInChat) {
			LOGD("AndroidPortAdditions::performSkip: dialog part playing");
			keyCode = Common::KEYCODE_SPACE;
			ascii = Common::ASCII_SPACE;
		} else {
			keyCode = Common::KEYCODE_ESCAPE;
			ascii = Common::ASCII_ESCAPE;

		}
		// Push the key event to the system
		e1.kbd.keycode = keyCode;
		e1.kbd.ascii = ascii;
		e1.type = Common::EVENT_KEYDOWN;
		android->forceEvent(e1);

		e2.kbd.keycode = keyCode;
		e2.kbd.ascii = ascii;
		e2.type = Common::EVENT_KEYUP;
		android->forceEvent(e2);
	} else if (mGameType == GAME_TYPE_T7G) {
		g_engine->fastForward(true);
	}
}

void AndroidPortAdditions::performRevealItems() {
	LOGD("AndroidPortAdditions::performRevealItems()");

	mLastTimeRevealItemsPressed = AndroidPortUtils::getTimeOfDayMillis();

// Obtain hotspots
	uint16 maxHotspots = 100;
	Hotspot hotspots[maxHotspots];
	uint16 count = mHitAreaHelper.getAllInteractionHotspots(hotspots,
			maxHotspots);

	AndroidBitmap* indicator = getBitmap("touch_indicator.png");
	AndroidBitmap* indicator2 = NULL;
	AndroidBitmap* indicator3 = NULL;

	if (mGameType == GAME_TYPE_T7G) {
		indicator2 = getBitmap("touch_indicator_2.png");
		indicator3 = getBitmap("touch_indicator_3.png");
	}

	AndroidBitmap* currentIndicator;

// Draw indicators on all of them with alphas
	for (int i = 0; i < count; ++i) {

		DrawablePtr hotspotDrawable(new Drawable), actionDrawable(new Drawable);
		DrawablePtr hotspotRectDrawable(NULL);
		if (mGameType == GAME_TYPE_T7G) {
			// Touch indicators for T7G

			if (hotspots[i].mHotspotType == HOTSPOT_TYPE_T7G_PUZZLE_1) {
				currentIndicator = indicator;
			} else if (hotspots[i].mHotspotType == HOTSPOT_TYPE_T7G_PUZZLE_2) {
				currentIndicator = indicator2;
			} else {
				// Check if we're in T7G map
				Groovie::GroovieEngine* groovie =
				(Groovie::GroovieEngine*) g_engine;
				if (groovie->getNavigationState() == MAP_SHOWN) {
					// On map, show both the indicator and the frame
					currentIndicator = indicator3;
				} else {
					// Outside map, show only the frame
					hotspotDrawable = NULL;
				}

				// Draw a frame
				hotspotRectDrawable.reset(new Drawable);
			}
		} else {
			// Other game types
			currentIndicator = indicator;
		}

		generateHotspotIndicatorDrawables(currentIndicator, hotspots[i], NULL,
				hotspotDrawable, actionDrawable, hotspotRectDrawable);

		if (hotspotRectDrawable != NULL) {

			// Create a fade in animation
			shared_ptr<AlphaAnimation> fadeInAnimation(new AlphaAnimation());
			fadeInAnimation->setDuration(REVEAL_ITEMS_FADEIN_DURATION);
			fadeInAnimation->setStartAlpha(0);
			fadeInAnimation->setEndAlpha(HOTSPOT_RECT_END_ALPHA);
			fadeInAnimation->setInterpolator(
					InterpolatorPtr(new DeccelerateInterpolator));

			// Create a stay animation
			shared_ptr<AlphaAnimation> stayAnimation(new AlphaAnimation());
			stayAnimation->setDuration(REVEAL_ITEMS_STAY_DURATION);
			stayAnimation->setStartAlpha(HOTSPOT_RECT_END_ALPHA);
			stayAnimation->setEndAlpha(HOTSPOT_RECT_END_ALPHA);

			// Create a fade out animation
			shared_ptr<AlphaAnimation> fadeOutAnimation(new AlphaAnimation());
			fadeOutAnimation->setDuration(REVEAL_ITEMS_FADEOUT_DURATION);
			fadeOutAnimation->setStartAlpha(HOTSPOT_RECT_END_ALPHA);
			fadeOutAnimation->setEndAlpha(0);
			fadeOutAnimation->setInterpolator(
					InterpolatorPtr(new AccelerateInterpolator));

			// Create a sequence
			shared_ptr<SequenceAnimationComposite> revealItemsAnimation(
					new SequenceAnimationComposite);
			revealItemsAnimation->addAnimation(fadeInAnimation);
			revealItemsAnimation->addAnimation(stayAnimation);
			revealItemsAnimation->addAnimation(fadeOutAnimation);
			revealItemsAnimation->start(AndroidPortUtils::getTimeOfDayMillis());

			hotspotRectDrawable->setAnimation(revealItemsAnimation);
			mAnimatedDrawables.push_back(hotspotRectDrawable);
		}

		if (hotspotDrawable != NULL) {

			// For T7G, we set a fixed width for the hotspots drawable only on "reveal items", so we do it here
			if (mGameType == GAME_TYPE_T7G)
			hotspotDrawable->setWidth(TOUCH_INDICATOR_W);

			// Create a fade in animation
			shared_ptr<AlphaAnimation> fadeInAnimation(new AlphaAnimation());
			fadeInAnimation->setDuration(REVEAL_ITEMS_FADEIN_DURATION);
			fadeInAnimation->setStartAlpha(0);
			fadeInAnimation->setEndAlpha(1);
			fadeInAnimation->setInterpolator(
					InterpolatorPtr(new DeccelerateInterpolator));

			// Create a stay animation
			shared_ptr<AlphaAnimation> stayAnimation(new AlphaAnimation());
			stayAnimation->setDuration(REVEAL_ITEMS_STAY_DURATION);
			stayAnimation->setStartAlpha(1);
			stayAnimation->setEndAlpha(1);

			// Create a fade out animation
			shared_ptr<AlphaAnimation> fadeOutAnimation(new AlphaAnimation());
			fadeOutAnimation->setDuration(REVEAL_ITEMS_FADEOUT_DURATION);
			fadeOutAnimation->setStartAlpha(1);
			fadeOutAnimation->setEndAlpha(0);
			fadeOutAnimation->setInterpolator(
					InterpolatorPtr(new AccelerateInterpolator));

			// Create a sequence
			shared_ptr<SequenceAnimationComposite> revealItemsAnimation(
					new SequenceAnimationComposite);
			revealItemsAnimation->addAnimation(fadeInAnimation);
			revealItemsAnimation->addAnimation(stayAnimation);
			revealItemsAnimation->addAnimation(fadeOutAnimation);
			revealItemsAnimation->start(AndroidPortUtils::getTimeOfDayMillis());

			hotspotDrawable->setAnimation(revealItemsAnimation);
			mAnimatedDrawables.push_back(hotspotDrawable);
		}
	}
}

void AndroidPortAdditions::toggleBottomPanel() {
	LOGD("AndroidPortAdditions::toggleBottomPanel: ");

	bool previousMode = g_engine->isBottomToolbarAppearing();
	g_engine->setFullScreenMode(!previousMode);
}

void AndroidPortAdditions::performInventoryScroll(bool up) {
	if (up) {
		pushClickEvent(INVENTORY_UP_ARROW_X, INVENTORY_UP_ARROW_Y);
	} else {
		pushClickEvent(INVENTORY_DOWN_ARROW_X, INVENTORY_DOWN_ARROW_Y);

	}
}

void AndroidPortAdditions::copyPixelsBetweenSurfaces(Surface* src, Surface* dst,
		Rect srcRect, Point dstPoint, bool startFromBottom) {
//	LOGD("AndroidPortAdditions::copyPixelsBetweenSurfaces: src: %d %d %d %d dst: %d %d startFromBottom: %d", srcRect.left, srcRect.top, srcRect.right, srcRect.bottom, dstPoint.x, dstPoint.y, startFromBottom);

	byte* srcPixels = (byte*) src->getPixels();
	byte* dstPixels = (byte*) dst->getPixels();

	byte bytesPerPixel = src->format.bytesPerPixel;

// Check that the pixel formats match (only in terms of bytes per pixel)
	if (dst->format.bytesPerPixel != bytesPerPixel) {
		LOGE(
				"AndroidPortAdditions::copyPixelsBetweenSurfaces: pixel formats do not match, pixels per byte: %d and %d",
				bytesPerPixel, dst->format.bytesPerPixel);
		return;
	}

	for (uint16 y = 0; y < srcRect.height(); ++y) {
		// Decide whether to count the rows from bottom or from top
		uint16 rowIndex = startFromBottom ? (srcRect.height() - 1 - y) : y;

		uint16 srcRow = srcRect.top + rowIndex;
		uint16 dstRow = dstPoint.y + rowIndex;

		byte* srcPtr = srcPixels + srcRow * src->pitch
		+ srcRect.left * bytesPerPixel;
		byte* dstPtr = dstPixels + dstRow * dst->pitch
		+ dstPoint.x * bytesPerPixel;

		memcpy(dstPtr, srcPtr, srcRect.width() * bytesPerPixel);
	}
}

void AndroidPortAdditions::onGameEngineTick()
{
	if (mGameType == GAME_TYPE_IHNM)
	{
		// HACK: Extra possible place for saving the game if needed.
		// This is done to allow the game to save even if for some reason the surface was destroyed but the game is not paused yet (Android edge case)
		saveIfNeeded();
	}
}

void AndroidPortAdditions::beforeDrawTextureToScreen(Surface* gameSurface) {
// Debugging
#ifdef ANDROID_PORT_DUMP_SCREEN

	if (mDumpPixelsToFile) {
		AndroidPortUtils::dumpBytesToFile((byte*) gameSurface->getPixels(),
				gameSurface->h * gameSurface->pitch
				* gameSurface->format.bytesPerPixel,
				"mnt/sdcard/gamescreen.clut8");
		mDumpPixelsToFile = false;
	}
#endif
#ifdef ANDROID_PORT_DUMP_CURSOR
	if (mDumpPixelsToFile) {
		AndroidPortUtils::dumpBytesToFile((byte*) mCursorBitmap->getPixels(),
				mCursorBitmap->getWidth() * mCursorBitmap->getHeight() * 2,
				"mnt/sdcard/gamecursor.rgb5551");

		LOGD(
				"AndroidPortAdditions::beforeDrawTextureToScreen: dumping cursor w %d h %d",
				mCursorBitmap->getWidth(), mCursorBitmap->getHeight());

		AndroidPortUtils::dumpBytesToFile(
				(byte*) CursorHelper::instance()->getCursorPalette(),
				CursorHelper::instance()->getCursorPaletteSize(),
				"mnt/sdcard/cursorpalette.rgb5551");
		mDumpPixelsToFile = false;
	}
#endif

	if (isSimonGame() || getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM) {

		// Check bottom toolbar
		checkBottomToolbar(gameSurface);

		// Check chat mode
		checkGameInChat(gameSurface);

		if (isSimonGame()) {
			// Check postcard screen
			checkGameInPostcard(gameSurface);

			// Identify the current game area
			GameArea previousArea = mCurrentGameArea;
			mCurrentGameArea = GameAreaIdentifier::identifyGameArea(
					(byte*) gameSurface->getPixels(),
					g_engine->getCurrentAreaId());

			// Workaround for Simon1 puddle save bug
			if (mCurrentGameArea == PUDDLE_WATER) {
				mShouldPerformSimon1PuddleWorkaround = true;
			}

			// If we were on hebrew title and it's gone, notify
			if (previousArea == HEBREW_TITLE
					&& mCurrentGameArea != HEBREW_TITLE) {
				JNI::gameEventJNIToJava(GAME_EVENT_HEBREW_TITLE_GONE);
			}
		}

		bool shouldMoveBlackPanel;
		if (mGameType == GAME_TYPE_IHNM) {
			shouldMoveBlackPanel = g_engine->isStatusBarAppearing();
		} else {
			shouldMoveBlackPanel = mBottomToolbarAppearing;
		}

		// FOR FOTAQ - additional check for a black pixel row
		if (mGameType == GAME_TYPE_FOTAQ && shouldMoveBlackPanel) {

			byte* rowPtr = (byte*) gameSurface->getPixels()
			+ (BLACK_PANEL_END_Y - 1) * gameSurface->pitch;
			byte* rowEnd = rowPtr
			+ gameSurface->w * gameSurface->format.bytesPerPixel;

			// Check if all bytes are 0 (applies to all pixel formats)
			while (rowPtr < rowEnd) {

				if (*rowPtr != 0) {
					// Byte != 0, so pixel is not black
					shouldMoveBlackPanel = false;
					break;
				}

				++rowPtr;
			}
		}

		if (shouldMoveBlackPanel && !mClassicMode) {
			// Move the middle black panel to the top.
			// This will also generate the modified game pixels.
			moveBlackPanel(gameSurface);

			mShouldUseModifiedGamePixels = true;

		} else {
			mShouldUseModifiedGamePixels = false;
		}
	}

// Auto load if needed
	if (isSimonGame())
	autoloadBehaviorSimon();

	loadIfNeeded();
	saveIfNeeded();

// Add reveal items animation if needed
	if (mShouldPerformRevealItems) {
		performRevealItems();
		mShouldPerformRevealItems = false;
	}

// All logic related to touch events in the game itself
	gameTouchBehavior();
}

void AndroidPortAdditions::autoloadBehaviorSimon() {
// Logic for auto-load behavior
	if (mAutoLoadSlot != -1) {
		// Spam "skip" events every few frames in case we are in auto-load.
		// This is to skip the starting animation and get fast into a state where loading can be done.
		if (mAutoLoadSpamSkipCounter % 5 == 0) {
			performSkip(false);
		}

		++mAutoLoadSpamSkipCounter;

		// If we got the botton toolbar, and the mouse is visible, we are after the opening movie.
		// Auto-load the slot, and get out of the auto-load state
		if (mBottomToolbarAppearing && mMouseVisible) {

			// SIMON 2 behavior - extra delay after introduction ends.
			if (getGameType() == GAME_TYPE_SIMON2) {
				uint64 currentTime = AndroidPortUtils::getTimeOfDayMillis();
				if (mAutoLoadIntroEndTimestamp == 0) {
					mAutoLoadIntroEndTimestamp = currentTime;
				}

				if (currentTime
						- mAutoLoadIntroEndTimestamp< SIMON2_LOAD_DELAY_AFTER_INTRO) {
					return;
				}
			}

			// We set the ending condition before performing the load, in order to avoid recursion
			// (because of the way the AGOS engine is implemented - loading results in direct screen update).
			int32 slotToLoadNow = mAutoLoadSlot;
			mAutoLoadSlot = -1;

			if (loadGame(slotToLoadNow)) {
				JNI::gameEventJNIToJava(GAME_EVENT_LOAD_SUCCESS);
			} else {
				JNI::gameEventJNIToJava(GAME_EVENT_LOAD_FAILURE);
				LOGE(
						"AndroidPortAdditions::autoloadBehaviorSimon: Error auto-loading slot %d",
						slotToLoadNow);
			}
		}
	}
}

void AndroidPortAdditions::loadIfNeeded() {
// Load chosen slot if needed
	if (mSlotToLoad != -1) {
		int32 slotToLoadNow = mSlotToLoad;
		mSlotToLoad = -1;

		// Check conditions for loading
		if (mBottomToolbarAppearing && mMouseVisible) {

			if (!loadGame(slotToLoadNow)) {
				JNI::gameEventJNIToJava(GAME_EVENT_LOAD_FAILURE);
				LOGE(
						"AndroidPortAdditions::beforeDrawTextureToScreen: Error loading slot %d",
						slotToLoadNow);
			} else {
				JNI::gameEventJNIToJava(GAME_EVENT_LOAD_SUCCESS);
			}
		} else {
			JNI::gameEventJNIToJava(GAME_EVENT_LOAD_FAILURE);
			LOGD(
					"AndroidPortAdditions::beforeDrawTextureToScreen: could not load at this time");
		}
	}
}

void AndroidPortAdditions::saveIfNeeded() {
// Check if there is a request to save
	if (mSlotToSave == -1) {
		return;
	}

	if (mForceSaveLoad) {
		int32 slotToSaveNow = mSlotToSave;
		mSlotToSave = -1;

		// SIMON1: Some saves should be prevented at all costs, because of engine\script bugs
		if (getGameType() == GAME_TYPE_SIMON1) {
			if (mCurrentGameArea == GOBLIN_FORTRESS_WITH_PAPER_ON_FLOOR) {
				JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
				return;
			}

			if (mShouldPerformSimon1PuddleWorkaround) {
				if (mCurrentGameArea == PUDDLE_AFTER_WATER
						|| mCurrentGameArea == BEFORE_PUDDLE) {
					JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
					return;
				}
			}

			if (mCurrentGameArea == PUDDLE_LAKE_MIDDLE) {
				JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
				return;
			}
		}

		if (!saveGame(slotToSaveNow)) {
			JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
			LOGE("AndroidPortAdditions::saveIfNeeded: Error saving slot %d",
					slotToSaveNow);
		} else {
			JNI::gameEventJNIToJava(GAME_EVENT_SAVE_SUCCESS);
		}

		return;
	}

// Behaviors for different games
	if (mGameType == GAME_TYPE_SIMON1) {
		saveProtectionSimon1();
		return;
	}

	if (mGameType == GAME_TYPE_SIMON2) {
		saveProtectionSimon2();
		return;
	}

	int32 slotToSaveNow = mSlotToSave;
	mSlotToSave = -1;

	if (checkSaveConditions() && saveGame(slotToSaveNow)) {
		JNI::gameEventJNIToJava(GAME_EVENT_SAVE_SUCCESS);
	} else {
		JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
		LOGE("AndroidPortAdditions::saveIfNeeded: Error saving slot %d",
				slotToSaveNow);
	}
}

void AndroidPortAdditions::saveProtectionSimon1() {

	uint64 currentTime = AndroidPortUtils::getTimeOfDayMillis();
// Check saving conditions for the first time
	if (!mSaveInProgress) {
		if (!checkSaveConditions()) {
			resetSaveState();
			JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
			LOGD(
					"AndroidPortAdditions::saveProtectionSimon1: could not save at this time");

			return;
		}

		mSaveInProgress = true;
		mSaveStartTimestamp = currentTime;

		// Stop Simon from walking when beginning saving
		g_engine->stopWalking();
	}

// Check if the game idle counter ticked since saving started.
	if (mGameIdleCountWhileSaving) {
		// If so, check conditions again and then save if possible
		if (checkSaveConditions()) {
			int32 slotToSaveNow = mSlotToSave;
			mSlotToSave = -1;
			if (!saveGame(slotToSaveNow)) {
				JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
				LOGE(
						"AndroidPortAdditions::saveProtectionSimon1: Error saving slot %d",
						slotToSaveNow);
			} else {
				JNI::gameEventJNIToJava(GAME_EVENT_SAVE_SUCCESS);
			}
		} else {
			JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
			LOGD(
					"AndroidPortAdditions::saveProtectionSimon1: could not save at this time");
		}

		resetSaveState();
	}

// Check if the timeout has passed, and fail if it did
	if (currentTime - mSaveStartTimestamp >= SIMON1_SAVE_TIMEOUT) {
		resetSaveState();
		JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
		LOGD(
				"AndroidPortAdditions::saveProtectionSimon1: could not save at this time");
	}
}

void AndroidPortAdditions::saveProtectionSimon2() {

	uint64 currentTime = AndroidPortUtils::getTimeOfDayMillis();
// Check saving conditions for the first time
	if (!mSaveInProgress) {
		if (!checkSaveConditions()) {
			resetSaveState();
			JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
			LOGD(
					"AndroidPortAdditions::saveProtectionSimon2: could not save at this time");

			return;
		}

		mSaveInProgress = true;
		mSaveStartTimestamp = currentTime;

		// Stop Simon from walking when beginning saving
		g_engine->stopWalking();
	}

// Check if the game cancelled idle since saving
	if (mGameCancelledIdleWhileSaving) {
		// Can't save at this time, somethin happens in the game
		resetSaveState();
		JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
		LOGD(
				"AndroidPortAdditions::saveProtectionSimon2: could not save at this time");
		return;
	}

// Check if the save test time has passed
	if (currentTime - mSaveStartTimestamp >= SIMON2_SAVE_TEST_TIME) {
		// If so, check conditions again and then save if possible
		if (checkSaveConditions()) {
			int32 slotToSaveNow = mSlotToSave;
			mSlotToSave = -1;
			if (!saveGame(slotToSaveNow)) {
				JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
				LOGE(
						"AndroidPortAdditions::saveProtectionSimon2: Error saving slot %d",
						slotToSaveNow);
			} else {
				JNI::gameEventJNIToJava(GAME_EVENT_SAVE_SUCCESS);
			}
		} else {
			JNI::gameEventJNIToJava(GAME_EVENT_SAVE_FAILURE);
			LOGD(
					"AndroidPortAdditions::saveProtectionSimon2: could not save at this time");
		}

		resetSaveState();
	}

}

void AndroidPortAdditions::resetSaveState() {
	mSlotToSave = -1;
	mSaveInProgress = false;
	mGameIdleCountWhileSaving = false;
	mGameCancelledIdleWhileSaving = false;
}

void AndroidPortAdditions::checkGameInChat(Surface* gameSurface) {

	bool gameInChat;

	if (getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM) {
		gameInChat = g_engine->isInChatMode();
	} else if (isSimonGame()) {
		if (!mMouseVisible || mBottomToolbarAppearing) {
			gameInChat = false;
		} else {
			//
			// check if the game is in chat mode by checking if a certain line has only black pixels
			//

			// Get the row pointer
			byte* rowPtr = (byte*) gameSurface->getPixels()
			+ CHAT_MODE_CHECK_Y * gameSurface->pitch;
			byte* rowEnd = rowPtr
			+ gameSurface->w * gameSurface->format.bytesPerPixel;

			// Check if all bytes are 0 (applies to all pixel formats)
			gameInChat = true;
			while (rowPtr < rowEnd) {

				if (*rowPtr) {
					// Byte != 0, so pixel is not black
					gameInChat = false;
					return;
				}

				++rowPtr;
			}
		}
	}

// Check if we got out of chat mode - if so, reposition the cursor in the game area.
// This is done to prevent false highlighting of a chat choice in the next chat screen.
	if (mGameInChat == true && gameInChat == false) {
		pushScrollEvent(0, 0);
	}

// Check if we got into chat mode - if so, set the default line selection to 1
	if (mGameInChat == false && gameInChat == true) {
		// Get all the chat hotspots
		Point points[10];
		uint16 count = mHitAreaHelper.getAllChatHotspots(points, 10);

		if (count > 0) {
			Point first = points[0];
			pushScrollEvent(first.x, first.y);

			mSelectedChatRow = 1;
		}

	}

	mGameInChat = gameInChat;
}

void AndroidPortAdditions::checkGameInPostcard(Surface* gameSurface) {
//
// check if the game is in the postcard screen by querying certain pixels
//

// Get the row pointer
	byte* rowPtr = (byte*) gameSurface->getPixels()
	+ POSTCARD_WINDOW_CHECK_Y * gameSurface->pitch;

// Check for a certain pixel pattern
	if (getGameType() == GAME_TYPE_SIMON1) {
		if (rowPtr[65] == 228 && rowPtr[75] == 243 && rowPtr[79] == 254
				&& rowPtr[91] == 227 && rowPtr[133] == 225 && rowPtr[200] == 225
				&& rowPtr[254] == 228) {
			mGameInPostcard = true;
		} else {
			mGameInPostcard = false;
		}
	} else if (getGameType() == GAME_TYPE_SIMON2) {
		if (rowPtr[65] == 0xEB && rowPtr[67] == 0xDB && rowPtr[100] == 0xDB
				&& rowPtr[126] == 0xEB && rowPtr[200] == 0xE4
				&& rowPtr[239] == 0xE4 && rowPtr[253] == 0xDB) {
			mGameInPostcard = true;
		} else {
			mGameInPostcard = false;
		}
	}
}

void AndroidPortAdditions::checkBottomToolbar(Surface* gameSurface) {

	if (isSimonGame()) {
//
// check if the game has inventory by querying certain pixels
//

// Get the row pointer
		byte* rowPtr = (byte*) gameSurface->getPixels()
		+ BOTTOM_TOOLBAR_CHECK_Y * gameSurface->pitch;

// Check for a certain pixel pattern

		if (getGameType() == GAME_TYPE_SIMON1) {
			if (rowPtr[1] == 240 && rowPtr[25] == 247 && rowPtr[50] == 240
					&& rowPtr[75] == 247 && rowPtr[100] == 250
					&& rowPtr[125] == 240) {
				mBottomToolbarAppearing = true;
			} else {
				mBottomToolbarAppearing = false;
			}
		}

		if (getGameType() == GAME_TYPE_SIMON2) {

			if (rowPtr[0] == 0xfa && rowPtr[17] == 0xf8 && rowPtr[69] == 0xf9
					&& rowPtr[123] == 0xf7 && rowPtr[188] == 0xf8
					&& rowPtr[254] == 0xf7) {
				mBottomToolbarAppearing = true;
			} else {
				mBottomToolbarAppearing = false;
			}
		}
	}

	if (getGameType() == GAME_TYPE_FOTAQ || getGameType() == GAME_TYPE_IHNM) {

		mBottomToolbarAppearing = (g_engine->isBottomToolbarAppearing());
	}
}

void AndroidPortAdditions::moveBlackPanel(Surface* gameSurface) {
//	LOGD("AndroidPortAdditions::moveBlackPanel: surface: w %d h %d pitch %d", gameSurface->w, gameSurface->h, gameSurface->pitch);

//
// In order to move the black panel, copy it to a temp pixel buffer, move the whole screen down,
// then copy the black panel to the top.
//

	uint16 panelHeight = BLACK_PANEL_END_Y - BLACK_PANEL_START_Y;

// Allocate the modified pixels if needed
	if (mModifiedGameSurface.getPixels() == NULL) {
		mModifiedGameSurface.create(GAME_SCREEN_WIDTH, GAME_SCREEN_HEIGHT,
				gameSurface->format);
	}

// Copy the black panel to the top
	Rect srcRect;
	srcRect.left = 0;
	srcRect.top = BLACK_PANEL_START_Y;
	srcRect.setWidth(GAME_SCREEN_WIDTH);
	srcRect.setHeight(panelHeight);

	Point dstPoint;
	dstPoint.x = 0;
	dstPoint.y = 0;

	copyPixelsBetweenSurfaces(gameSurface, &mModifiedGameSurface, srcRect,
			dstPoint);

// Copy the upper game screen to the previous black panel location (move down)
	srcRect.left = 0;
	srcRect.top = 0;
	srcRect.setWidth(GAME_SCREEN_WIDTH);
	srcRect.setHeight(BLACK_PANEL_START_Y);

	dstPoint.x = 0;
	dstPoint.y = panelHeight;

	copyPixelsBetweenSurfaces(gameSurface, &mModifiedGameSurface, srcRect,
			dstPoint);

// Copy the remains of the screen to the same position
	srcRect.left = 0;
	srcRect.top = BLACK_PANEL_END_Y;
	srcRect.setWidth(GAME_SCREEN_WIDTH);
	srcRect.setHeight(GAME_SCREEN_HEIGHT - BLACK_PANEL_END_Y);

	dstPoint.x = 0;
	dstPoint.y = BLACK_PANEL_END_Y;

	copyPixelsBetweenSurfaces(gameSurface, &mModifiedGameSurface, srcRect,
			dstPoint);
}

bool AndroidPortAdditions::shouldUseModifiedGamePixels() {
	if (mModifiedGameSurface.getPixels() == NULL) {
		// Not initialized yet
		return false;
	}

	return mShouldUseModifiedGamePixels;
}

byte*
AndroidPortAdditions::getModifiedGamePixels() {
	return (byte*) mModifiedGameSurface.getPixels();
}

void AndroidPortAdditions::setClassicMode(bool classicMode) {
	LOGD("AndroidPortAdditions::setClassicMode: %d", classicMode);

	mClassicMode = classicMode;
}

bool AndroidPortAdditions::getClassicMode() {
	return mClassicMode;
}

void AndroidPortAdditions::translateTouchCoordinates(int& x, int& y) {
//	LOGD("AndroidPortAdditions::translateTouchCoordinates: before: %d %d", x, y);

// Check if we need to translate at that position
	if (mShouldUseModifiedGamePixels && y <= mTouchTranslationLowerBound) {
		// Translate the real coordinates into our fake coordinates (because we moved screen parts)
		y -= mTouchTranslationOffset;
		if (y < 0) {
			y = 0;
		}
	}

//	LOGD("AndroidPortAdditions::translateTouchCoordinates: after: %d %d", x, y);
}

AndroidBitmap*
AndroidPortAdditions::getActionIcon(uint16 action) {
	switch (action) {
		case ACTION_WALK:
		return getBitmap("walk.png");
		case ACTION_LOOK:
		return getBitmap("look.png");
		case ACTION_OPEN:
		return getBitmap("open.png");
		case ACTION_MOVE:
		return getBitmap("move.png");
		case ACTION_CONSUME:
		return getBitmap("consume.png");
		case ACTION_PICK:
		return getBitmap("pick.png");
		case ACTION_CLOSE:
		return getBitmap("close.png");
		case ACTION_USE:
		return getBitmap("use.png");
		case ACTION_TALK:
		return getBitmap("talk.png");
		case ACTION_REMOVE:
		return getBitmap("remove.png");
		case ACTION_WEAR:
		return getBitmap("wear.png");
		case ACTION_GIVE:
		return getBitmap("give.png");
		default:
		return NULL;
	}
}

void AndroidPortAdditions::preventIdleMode() {
	LOGD("AndroidPortAdditions::preventIdleMode()");

	g_engine->preventIdleMode();
}

bool AndroidPortAdditions::checkSaveConditions() {

// Specialized conditions for Simon 1
#if CURRENT_GAME_TYPE == GAME_TYPE_SIMON1
// Override condition - never allow saving in goblin fortress with paper on floor due to engine bug
// http://wiki.scummvm.org/index.php/AGOS/Bugs
	if (mCurrentGameArea == GOBLIN_FORTRESS_WITH_PAPER_ON_FLOOR) {
		return false;
	}

// Override condition - never allow saving in puddle area after water was poured
	if (mShouldPerformSimon1PuddleWorkaround) {
		if (mCurrentGameArea == PUDDLE_AFTER_WATER
				|| mCurrentGameArea == BEFORE_PUDDLE) {
			return false;
		}
	}

// Never allow saving in the middle of the lake
	if (mCurrentGameArea == PUDDLE_LAKE_MIDDLE) {
		return false;
	}

	int32 timerEvents = g_engine->getTimerEventCount();
	return mBottomToolbarAppearing && mMouseVisible && timerEvents <= 1;
#endif

// Specialized conditions for Simon2
#if CURRENT_GAME_TYPE == GAME_TYPE_SIMON2
	if (g_engine->getGameIdleFlag() == false || g_engine->isInScrollingMap()) {
		return false;
	}

	int32 timerEvents = g_engine->getTimerEventCount();
	return mBottomToolbarAppearing && mMouseVisible && timerEvents <= 1;
#endif

	// Specialized conditions for FOTAQ
#if CURRENT_GAME_TYPE == GAME_TYPE_FOTAQ

	return ((Queen::QueenEngine*)g_engine)->canSaveGameStateCurrently();
#endif

#if CURRENT_GAME_TYPE == GAME_TYPE_IHNM
	return (mGameInChat == false);
#endif

	return true;
}

bool AndroidPortAdditions::checkLoadConditions() {
	return mBottomToolbarAppearing && mMouseVisible;
}

void AndroidPortAdditions::forceAutosave()
{
	JNI::gameEventJNIToJava(GAME_EVENT_SET_FORCE_AUTOSAVING);
	mSlotToSave = 1;
	saveIfNeeded();
}

void AndroidPortAdditions::gameTouchEvent(int16 x, int16 y, int16 origX,
		int16 origY, TouchState touchState) {

	if (mDisallowNextTouchEvent) {
		// Ignore the current touch event - workaround for scenarios where we handle a gesture and want to ignore its touch components.
		mDisallowNextTouchEvent = false;
		return;
	}

	mTouchEventMutex->lock();

	int gameTouchX = x * mGameToDisplayRatioW;
	int gameTouchY = y * mGameToDisplayRatioH;

// Check for fling on inventory
	if (touchState == FLING) {
		int gameOrigX = origX * mGameToDisplayRatioW;
		int gameOrigY = origY * mGameToDisplayRatioH;

		if (isSimonGame() && gameTouchX >= INVENTORY_FLING_MIN_X
				&& gameOrigX >= INVENTORY_FLING_MIN_X
				&& gameTouchX <= INVENTORY_FLING_MAX_X
				&& gameOrigX <= INVENTORY_FLING_MAX_X
				&& gameTouchY >= INVENTORY_FLING_MIN_Y
				&& gameOrigY >= INVENTORY_FLING_MIN_Y) {

			// Check fling direction
			bool up = (gameOrigY <= gameTouchY);
			performInventoryScroll(up);

			// We don't deal with this event any more, and ignore the next "UP"
			touchState = NONE;
			mDisallowNextTouchEvent = true;
		} else {
			// Fling was not on inventory, we treat it as a normal scroll
			touchState = DOWN;
		}
	}

	TouchEvent touchEvent;

	touchEvent.mTouchX = gameTouchX;
	touchEvent.mTouchY = gameTouchY;

// Adjust Y if it's inside the game area, because we move the black panel
	if ((isSimonGame() || getGameType() == GAME_TYPE_FOTAQ
					|| getGameType() == GAME_TYPE_IHNM) && !mGameInChat
			&& touchEvent.mTouchY <= BLACK_PANEL_END_Y) {
		touchEvent.mTouchY -= BLACK_PANEL_HEIGHT;
		if (touchEvent.mTouchY < 0) {
			touchEvent.mTouchY = 0;
		}
	}

// Check for any event inside the chat area, and reset the selected row if needed
	if (mGameInChat && touchEvent.mTouchY >= CHAT_HIT_AREAS_START_Y) {
		mSelectedChatRow = 0;
	}

// SIMON1: Adjust X to the right side if we're in the inventory arrow area.
// This is done to increase the arrow hit zone.
	if (getGameType() == GAME_TYPE_SIMON1 && mBottomToolbarAppearing
			&& touchEvent.mTouchY > BLACK_PANEL_END_Y
			&& touchEvent.mTouchX >= INVENTORY_SCROLL_ARROW_START_X) {
		touchEvent.mTouchX = INVENTORY_SCROLL_ARROW_END_X;
	}

	touchEvent.mTouchState = touchState;

// If the previous element we inserted is a DOWN event, replace it
	bool replaceLastEvent = false;
	if (!mTouchEventQueue.empty() && touchEvent.mTouchState == DOWN) {
		TouchEvent& previousEvent = mTouchEventQueue.back();
		if (previousEvent.mTouchState == DOWN) {
			previousEvent.mTouchX = touchEvent.mTouchX;
			previousEvent.mTouchY = touchEvent.mTouchY;
			replaceLastEvent = true;
		}
	}

	if (!replaceLastEvent) {
		mTouchEventQueue.push(touchEvent);
	}

	mTouchEventMutex->unlock();

}

void AndroidPortAdditions::outsideDragonCaveWorkaround(int16 &x, int16 &y) {

	LOGD("AndroidPortAdditions::outsideDragonCaveWorkaround: %d %d", x, y);

// Temporary workaround - make the boulder hit area bigger
	if (x >= OUTSIDE_DRAGON_CAVE_WORKAROUND_MIN_X
			&& x <= OUTSIDE_DRAGON_CAVE_WORKAROUND_MAX_X
			&& y >= OUTSIDE_DRAGON_CAVE_WORKAROUND_MIN_Y
			&& y <= OUTSIDE_DRAGON_CAVE_WORKAROUND_MAX_Y) {

		x = OUTSIDE_DRAGON_CAVE_WORKAROUND_TARGET_X;
		y = OUTSIDE_DRAGON_CAVE_WORKAROUND_TARGET_Y;
	}
}

void AndroidPortAdditions::chatArrowClick(bool up) {

	LOGD("AndroidPortAdditions::chatArrowClick: %d", up);

// Get all the chat hotspots
	Point points[10];
	uint16 count = mHitAreaHelper.getAllChatHotspots(points, 10);

// Adjust the selected row number.
// Chat hit areas are ordered from bottom to top.
	if (up) {
		--mSelectedChatRow;
		if (mSelectedChatRow <= 0) {
			mSelectedChatRow = count;
		}
	} else {
		++mSelectedChatRow;
		if (mSelectedChatRow > count) {
			mSelectedChatRow = 1;
		}
	}

// Scroll the game mouse to the selected row
	Point selected = points[mSelectedChatRow - 1];

	mTouchEventMutex->lock();
	TouchEvent touchEvent;
	touchEvent.mTouchState = DOWN;
	touchEvent.mTouchX = selected.x;
	touchEvent.mTouchY = selected.y;
	mTouchEventQueue.push(touchEvent);
	mTouchEventMutex->unlock();
}

void AndroidPortAdditions::chatButtonClick() {
// Get all the chat hotspots
	Point points[10];
	uint16 count = mHitAreaHelper.getAllChatHotspots(points, 10);

// Click the selected row
	Point selected;
	if (mSelectedChatRow <= 0 || mSelectedChatRow > count) {
		selected = points[0];
	} else {
		selected = points[mSelectedChatRow - 1];
	}

	pushClickEvent(selected.x, selected.y);

	mSelectedChatRow = 0;
}

bool AndroidPortAdditions::isOpenHouseGrid() {
// Check if we're in open house grid
	if (mGameType == GAME_TYPE_T7G) {
		Groovie::GroovieEngine* groovie = (Groovie::GroovieEngine*) g_engine;
		if (groovie->getNavigationState() == OPEN_HOUSE_SHOWN) {
			return true;
		}
	}

	return false;
}

uint32 AndroidPortAdditions::displayToGameCoordinatesX(uint displayX) {

}

uint32 AndroidPortAdditions::displayToGameCoordinatesY(uint displayY) {

}

uint32 AndroidPortAdditions::gameToDisplayCoordinatesX(uint gameX) {
// TODO
	return 0;
}

uint32 AndroidPortAdditions::gameToDisplayCoordinatesY(uint gameY) {
// TODO
	return 0;
}

void AndroidPortAdditions::enableOpenHouseMode() {
	LOGD("AndroidPortAdditions::enableOpenHouseMode: ");

	JNI::gameEventJNIToJava(GAME_EVENT_ENABLE_OPEN_HOUSE);
}

#endif
