package org.iphsoft.simon1;

import org.iphsoft.simon1.AndroidPortAdditions.GameType;
import org.iphsoft.simon1.stats.StatisticsManager;
import org.iphsoft.simon1.ui.ActivityAnimationUtil;
import org.iphsoft.simon1.util.BackgroundMusicPlayer;
import org.iphsoft.simon1.util.BitmapUtils;
import org.iphsoft.simon1.util.ImmersiveModeHelper;
import org.iphsoft.simon1.util.IntentUtils;
import org.iphsoft.simon1.util.Version;

import android.app.Activity;
import android.os.Bundle;
import android.text.Html;
import android.text.method.LinkMovementMethod;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.TextView;

import com.mojotouch.fotaq.R;

public class AboutActivity extends Activity {

	// Workaround for string resource limitations
	private static final String ABOUT_STRING_2 = "<br>Published and Developed by MojoTouch.<br>All Rights Reserved.<br><br>Lead Developer: Omer Gilad<br>UI graphics by Zach Sigal,<br><i>I &lt;M&gt; Duck</i>";
	
	private static final String ABOUT_STRING_2_IHNM = "<br>Published by DotEmu<br>All Rights Reserved.<br>";
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		mImmersiveModeHelper = new ImmersiveModeHelper(this);

		setContentView(R.layout.about);

		View bgFrame = findViewById(R.id.bgFrame);
		switch (AndroidPortAdditions.GAME_TYPE)
		{
		case SIMON1:
			BitmapUtils.setScaledBackground(this, bgFrame, R.drawable.bg);
			break;
		case SIMON2:
			BitmapUtils.setScaledBackground(this, bgFrame, R.drawable.bg_open);
			break;
		case T7G:
		case FOTAQ:
			View bg2 = findViewById(R.id.bg2);
			BitmapUtils.setScaledBackground(this, bgFrame, R.drawable.splash);
			BitmapUtils.setScaledBackground(this, bg2, R.drawable.bg);
			break;
		case IHNM:
			BitmapUtils.setScaledBackground(this, bgFrame, R.drawable.bg_settings2);
		}

		TextView txtAbout1 = (TextView) findViewById(R.id.txtAbout1);
		TextView txtAbout2 = (TextView) findViewById(R.id.txtAbout2);
		TextView txtAbout3 = (TextView) findViewById(R.id.txtAbout3);
		TextView txtAbout4 = (TextView) findViewById(R.id.txtAbout4);
		TextView txtAbout5 = (TextView) findViewById(R.id.txtAbout5);
		TextView txtAbout6 = (TextView) findViewById(R.id.txtAbout6);

		// Add the about string arguments
		String about1 = getString(R.string.about1, Version.getPackageVersion());
		txtAbout1.setText(about1);

		if (AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			txtAbout2.setText(Html.fromHtml(ABOUT_STRING_2_IHNM));
		}
		else
		{
			txtAbout2.setText(Html.fromHtml(ABOUT_STRING_2));
		}

		// Activate links
		txtAbout3.setMovementMethod(LinkMovementMethod.getInstance());
		txtAbout4.setMovementMethod(LinkMovementMethod.getInstance());
		txtAbout5.setMovementMethod(LinkMovementMethod.getInstance());
		txtAbout6.setMovementMethod(LinkMovementMethod.getInstance());

		// Set image links
		findViewById(R.id.imgMojotouch).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				IntentUtils.showBrowserApp(AboutActivity.this,
						getResources().getString(R.string.mojotouch_url));

			}
		});

		findViewById(R.id.imgScummVM).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				IntentUtils.showBrowserApp(AboutActivity.this,
						getResources().getString(R.string.scummvm_url));

			}
		});

		View imgWoodcock = findViewById(R.id.imgWoodcock);
		if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1)
		{
			imgWoodcock.setOnClickListener(new OnClickListener()
			{

				@Override
				public void onClick(View arg0)
				{
					IntentUtils.showBrowserApp(AboutActivity.this,
							getResources().getString(R.string.james_woodcock_url));

				}
			});
		} else
		{
			imgWoodcock.setVisibility(View.GONE);
		}

		View imgCdi = findViewById(R.id.imgCdi);
		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
		{
			imgCdi.setOnClickListener(new OnClickListener()
			{

				@Override
				public void onClick(View arg0)
				{
					IntentUtils.showBrowserApp(AboutActivity.this,
							getResources().getString(R.string.cdi_url));

				}
			});
		} else
		{
			imgWoodcock.setVisibility(View.GONE);
		}

		// Back button
		findViewById(R.id.btnBack).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				finish();

				if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1
						|| AndroidPortAdditions.GAME_TYPE == GameType.T7G
						|| AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ
						|| AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
				{
					ActivityAnimationUtil.makeActivityNullTransition(AboutActivity.this);
				} else if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON2)
				{
					ActivityAnimationUtil.makeActivityFadeTransition(AboutActivity.this);
				}
			}
		});

		// "rate us" button
		findViewById(R.id.btnRate).setOnClickListener(new OnClickListener()
		{

			@Override
			public void onClick(View arg0)
			{
				IntentUtils.startAppStoreActivity(AboutActivity.this, getPackageName());

			}
		});
	}

	@Override
	protected void onStart()
	{
		super.onStart();

		StatisticsManager.getStatistics().reportActivityStart(this);

		if (AndroidPortAdditions.HAS_BACKGROUND_MUSIC
				&& getIntent().getBooleanExtra(GameMenuActivity.EXTRA_MID_GAME, true) == false)
		{
			BackgroundMusicPlayer.addBackgroundMusicActivity();
		}
	}

	@Override
	public void onBackPressed()
	{
		super.onBackPressed();

		if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON1
				|| AndroidPortAdditions.GAME_TYPE == GameType.T7G
				|| AndroidPortAdditions.GAME_TYPE == GameType.FOTAQ
				|| AndroidPortAdditions.GAME_TYPE == GameType.IHNM)
		{
			ActivityAnimationUtil.makeActivityNullTransition(this);
		} else if (AndroidPortAdditions.GAME_TYPE == GameType.SIMON2)
		{
			ActivityAnimationUtil.makeActivityFadeTransition(this);
		}

	}

	@Override
	protected void onStop()
	{
		super.onStop();

		StatisticsManager.getStatistics().reportActivityStop(this);

		if (AndroidPortAdditions.HAS_BACKGROUND_MUSIC
				&& getIntent().getBooleanExtra(GameMenuActivity.EXTRA_MID_GAME, true) == false)
		{
			BackgroundMusicPlayer.removeBackgroundMusicActivity();
		}
	}

	@Override
	protected void onResume()
	{
		// TODO Auto-generated method stub
		super.onResume();

		mImmersiveModeHelper.supportFullScreenImmersiveMode();

	}

	private ImmersiveModeHelper mImmersiveModeHelper;

}
