package org.iphsoft.simon1;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.iphsoft.simon1.AndroidPortAdditions.GameType;
import org.iphsoft.simon1.stats.StatisticsManager;
import org.iphsoft.simon1.ui.ActivityAnimationUtil;
import org.iphsoft.simon1.ui.VideoActivity;
import org.iphsoft.simon1.util.BackgroundMusicPlayer;
import org.iphsoft.simon1.util.BitmapUtils;
import org.iphsoft.simon1.util.ImmersiveModeHelper;
import org.iphsoft.simon1.util.IntentUtils;
import org.iphsoft.simon1.util.MyLog;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.BaseAdapter;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.mojotouch.fotaq.R;

public class ExtrasActivity extends Activity {

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		mImmersiveModeHelper = new ImmersiveModeHelper(this);

		setContentView(R.layout.extras_activity);

		View bgFrame = findViewById(R.id.bgFrame);
		View bg2 = findViewById(R.id.bg2);

		if (AndroidPortAdditions.GAME_TYPE == GameType.T7G)
		{
			BitmapUtils.setScaledBackground(this, bgFrame, R.drawable.splash);
			BitmapUtils.setScaledBackground(this, bg2, R.drawable.bg);
		} else
		{
			BitmapUtils.setScaledBackground(this, bgFrame, R.drawable.bg);
		}

		try
		{
			mTopLevelExtraItems = prepareItems();
		} catch (Exception e)
		{
			e.printStackTrace();
			MyLog.e("ExtrasActivity: onCreate: error opening JSON: " + e);

			Toast.makeText(this, "Error opening extras", Toast.LENGTH_LONG).show();
			finish();
			return;
		}

		// Set up list
		mLvExtras = (ListView) findViewById(R.id.lvExtras);
		mAdapter = new ExtrasAdapter();
		mAdapter.setItems(mTopLevelExtraItems);
		mLvExtras.setAdapter(mAdapter);

		mLvExtras.setOnItemClickListener(new OnItemClickListener()
		{
			@Override
			public void onItemClick(AdapterView<?> arg0, View arg1, int pos, long arg3)
			{
				ExtraItem item = (ExtraItem) mAdapter.getItem(pos);
				switch (item.mType)
				{
				case VIDEO:
					// Open the video player
					String filename = AndroidPortAdditions.instance().getGameFileDir()
							+ File.separator + item.mContent;
					Intent videoIntent = new Intent(ExtrasActivity.this, VideoActivity.class);
					videoIntent.putExtra(VideoActivity.EXTRA_VIDEO_FILE, filename);
					startActivity(videoIntent);
					break;

				case MUSIC:
					// Stop background music the first time that any music is
					// selected
					mMusicItemSelected = true;
					if (AndroidPortAdditions.HAS_BACKGROUND_MUSIC)
						BackgroundMusicPlayer.removeBackgroundMusicActivity();
				case FILE:
					// Open the file in an external app
					File file = new File(AndroidPortAdditions.instance().getGameFileDir()
							+ File.separator + item.mContent);
					IntentUtils.openFile(ExtrasActivity.this, file);
					break;
				case LINK:
					IntentUtils.showBrowserApp(ExtrasActivity.this, item.mContent);
					break;
				case LIST:
					mAdapter.setItems(item.mSubList);
					mAdapter.notifyDataSetChanged();
					mCurrentSubListItem = item;
					break;
				}
			}
		});

		// Back button
		findViewById(R.id.btnBack).setOnClickListener(new OnClickListener()
		{
			@Override
			public void onClick(View arg0)
			{
				goBack();
			}
		});
	}

	@Override
	protected void onStart()
	{
		super.onStart();

		StatisticsManager.getStatistics().reportActivityStart(this);

		if (AndroidPortAdditions.HAS_BACKGROUND_MUSIC)
		{
			if (!mMusicItemSelected)
			{
				BackgroundMusicPlayer.addBackgroundMusicActivity();
			}
		}
	}

	@Override
	public void onBackPressed()
	{
		super.onBackPressed();

		ActivityAnimationUtil.makeActivityNullTransition(this);
	}

	@Override
	protected void onStop()
	{
		super.onStop();

		StatisticsManager.getStatistics().reportActivityStop(this);

		if (AndroidPortAdditions.HAS_BACKGROUND_MUSIC)
		{
			if (!mMusicItemSelected)
			{
				BackgroundMusicPlayer.removeBackgroundMusicActivity();
			}
		}
	}

	@Override
	protected void onResume()
	{
		super.onResume();

		mImmersiveModeHelper.supportFullScreenImmersiveMode();
	}

	private ExtraItem[] prepareItems() throws IOException, JSONException
	{
		// Open the JSON file
		InputStream is;
		if (AndroidPortAdditions.EXTRAS_EXTERNAL_JSON_FILE)
		{
			try
			{
				is = new FileInputStream(Environment.getExternalStorageDirectory().getPath()
						+ File.separator + "extras.json");
			} catch (Exception e)
			{
				Toast.makeText(this, "extras.json not found", Toast.LENGTH_SHORT).show();
				is = getResources().openRawResource(R.raw.extras);
			}
		} else
		{
			is = getResources().openRawResource(R.raw.extras);
		}

		byte[] buffer = new byte[1024 * 256];
		int offset = 0, count = 0;
		while ((count = is.read(buffer, offset, buffer.length - offset)) != -1)
		{
			offset += count;
		}

		is.close();

		String jsonString = new String(buffer, 0, offset);
		JSONObject mainObject = new JSONObject(jsonString);
		JSONArray mainArray = mainObject.getJSONArray("ExtrasList");

		return readListFromJson(mainArray, null);
	}

	private void goBack()
	{
		if (mCurrentSubListItem == null)
		{
			finish();
			ActivityAnimationUtil.makeActivityNullTransition(ExtrasActivity.this);
		} else if (mCurrentSubListItem.mParent == null)
		{
			mAdapter.setItems(mTopLevelExtraItems);
			mAdapter.notifyDataSetChanged();
			mCurrentSubListItem = null;
		} else
		{
			mAdapter.setItems(mCurrentSubListItem.mParent.mSubList);
			mAdapter.notifyDataSetChanged();
			mCurrentSubListItem = mCurrentSubListItem.mParent;
		}
	}

	private ExtraItem[] readListFromJson(JSONArray jsonArray, ExtraItem parent)
			throws JSONException
	{
		ExtraItem[] items = new ExtraItem[jsonArray.length()];

		for (int i = 0; i < jsonArray.length(); ++i)
		{
			JSONObject jsonObject = jsonArray.getJSONObject(i);

			ExtraItem item = new ExtraItem();
			item.mTitle = jsonObject.getString("title");
			item.mType = ExtraItemType.fromJSONType(jsonObject.getString("type"));
			if (item.mType == ExtraItemType.LIST)
			{
				item.mSubList = readListFromJson(jsonObject.getJSONArray("content"), item);
			} else
			{
				item.mContent = jsonObject.getString("content");
			}

			item.mParent = parent;

			items[i] = item;
		}

		return items;
	}

	private ListView mLvExtras;

	private enum ExtraItemType
	{
		VIDEO, MUSIC, FILE, LINK, LIST;

		public static ExtraItemType fromJSONType(String s) throws JSONException
		{
			if (s.equals("video"))
				return VIDEO;
			else if (s.equals("music"))
				return MUSIC;
			else if (s.equals("file"))
				return FILE;
			else if (s.equals("link"))
				return LINK;
			else if (s.equals("list"))
				return LIST;
			else
				throw new JSONException("invalid type: " + s);
		}
	}

	private class ExtraItem {
		public String mTitle;
		public String mContent;
		public ExtraItemType mType;
		public ExtraItem[] mSubList;
		public ExtraItem mParent = null;
	}

	private class ExtrasAdapter extends BaseAdapter {

		@Override
		public View getView(int position, View convertView, ViewGroup parent)
		{
			ExtraItem item = (ExtraItem) getItem(position);

			View view = convertView;
			if (view == null)
			{
				view = LayoutInflater.from(ExtrasActivity.this).inflate(R.layout.extras_list_row,
						null);
			}

			TextView txtExtra = (TextView) view.findViewById(R.id.txtExtra);
			txtExtra.setText(item.mTitle);

			return view;
		}

		@Override
		public long getItemId(int position)
		{
			return position;
		}

		@Override
		public Object getItem(int position)
		{
			// TODO Auto-generated method stub
			return mCurrentItems[position];
		}

		@Override
		public int getCount()
		{
			// TODO Auto-generated method stub
			return mCurrentItems.length;
		}

		public void setItems(ExtraItem[] items)
		{
			mCurrentItems = items;
		}

		private ExtraItem[] mCurrentItems;
	};

	private ExtrasAdapter mAdapter;
	private ExtraItem[] mTopLevelExtraItems;
	private ExtraItem mCurrentSubListItem = null;

	private boolean mMusicItemSelected = false;

	private ImmersiveModeHelper mImmersiveModeHelper;

}
