package org.iphsoft.simon1;

import java.util.ArrayList;
import java.util.List;

import org.iphsoft.simon1.stats.StatisticsManager;
import org.iphsoft.simon1.util.MyLog;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.R.integer;
import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;

import com.facebook.FacebookRequestError;
import com.facebook.Request;
import com.facebook.Request.Callback;
import com.facebook.Response;
import com.facebook.Session;
import com.facebook.Session.NewPermissionsRequest;
import com.facebook.SessionDefaultAudience;
import com.facebook.SessionState;
import com.facebook.UiLifecycleHelper;
import com.facebook.model.GraphObject;
import com.facebook.model.OpenGraphAction;
import com.mojotouch.fotaq.R;

public abstract class FacebookBaseActivity extends Activity {

	public enum PublishPermissionState
	{
		NEVER_ASKED, ACCEPTED, DECLINED, NOT_LOGGED_IN
	}

	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			mFacebookUiHelper = new UiLifecycleHelper(this, mStatusCallback);
			mFacebookUiHelper.onCreate(savedInstanceState);
		}
	}

	@Override
	protected void onResume()
	{
		super.onResume();

		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			// FROM FACEBOOK DOCS -
			// https://developers.facebook.com/docs/android/login-with-facebook/v2.2#step2
			//
			// For scenarios where the main activity is launched and user
			// session is not null, the session state change notification
			// may not be triggered. Trigger it if it's open/closed.
			Session session = Session.getActiveSession();
			if (session != null && (session.isOpened() || session.isClosed()))
			{
				onSessionStateChange(session, session.getState(), null);
			}

			mFacebookUiHelper.onResume();
		}
	}

	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		super.onActivityResult(requestCode, resultCode, data);

		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			MyLog.d("FacebookBaseActivity: onActivityResult: " + requestCode + " " + resultCode
					+ " " + data);

			mFacebookUiHelper.onActivityResult(requestCode, resultCode, data);
		}
	}

	@Override
	public void onPause()
	{
		super.onPause();

		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			mFacebookUiHelper.onPause();
		}
	}

	@Override
	public void onDestroy()
	{
		super.onDestroy();

		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			mFacebookUiHelper.onDestroy();
		}
	}

	@Override
	public void onSaveInstanceState(Bundle outState)
	{
		super.onSaveInstanceState(outState);

		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			mFacebookUiHelper.onSaveInstanceState(outState);
		}
	}

	protected boolean isFacebookLoggedIn()
	{
		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			Session session = Session.getActiveSession();
			if (session != null && session.isOpened())
			{
				return true;
			}

			return false;
		} else
		{
			return false;
		}
	}

	protected PublishPermissionState getPublishPermissionState()
	{
		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			Session session = Session.getActiveSession();

			// Make sure we are logged in
			if (session == null || !session.isOpened())
			{
				return PublishPermissionState.NOT_LOGGED_IN;
			}

			// Check if the accepted permissions contain publish_actions
			List<String> permissions = session.getPermissions();
			if (permissions.contains(PUBLISH_PERMISISONS))
			{
				return PublishPermissionState.ACCEPTED;
			}

			// Check if the declined permissions contain publish_actions
			List<String> declinedPermissions = session.getDeclinedPermissions();
			if (declinedPermissions.contains(PUBLISH_PERMISISONS))
			{
				return PublishPermissionState.DECLINED;
			}

			// If none of the lists contain it, consider it as never asked
			return PublishPermissionState.NEVER_ASKED;
		} else
		{
			return PublishPermissionState.NOT_LOGGED_IN;
		}
	}

	protected boolean requestPublishPermissions()
	{
		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			Session session = Session.getActiveSession();
			if (session == null)
			{
				return false;
			}

			NewPermissionsRequest request = new NewPermissionsRequest(this, PUBLISH_PERMISISONS);
			request.setDefaultAudience(SessionDefaultAudience.EVERYONE);
			request.setCallback(mNewPermissionsCallback);

			session.requestNewPublishPermissions(request);

			return true;
		} else
		{
			return false;
		}
	}

	protected boolean login()
	{
		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			Session session = Session.openActiveSession(this, true, mStatusCallback);

			if (session == null)
			{
				return false;
			}

			return true;
		} else
		{
			return false;
		}
	}

	/**
	 * Perform an async request to post an achievement for a user.
	 * 
	 * @param index
	 */
	protected void postNewAchievement(final int index, final boolean explicitShare,
			String userMessage)
	{
		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			mLastShareMessage = explicitShare ? userMessage : null;
			
			// Get the achievement object ID.
			String[] idArray = getResources().getStringArray(R.array.achievement_facebook_ids);
			String id = idArray[index];

			// Set the action
			OpenGraphAction action = OpenGraphAction.Factory.createForPost(EARN_ACTION_TYPE);
			action.setProperty(ACHIEVEMENT_PROPERTY, id);
			action.setProperty(ACHIEVEMENT_ID_PROPERTY, index);
			action.setProperty(EXPLICIT_SHARE_PROPERTY, explicitShare);

			if (explicitShare)
			{
				// Only for shares that are manually done by the user
				action.setExplicitlyShared(true);
				if (userMessage != null)
				{
					action.setProperty(USER_MESSAGE_PROPERTY, userMessage);
				}
			}

			// Set the request callback
			Request.Callback callback = new Callback()
			{

				@Override
				public void onCompleted(Response response)
				{
					// Notify the activity callback
					onAchievementPosted(index, explicitShare, response, mLastShareMessage);
				}
			};

			// Set the request and execute
			Request request = Request.newPostOpenGraphActionRequest(Session.getActiveSession(),
					action, callback);
			request.executeAsync();
		}
	}

	protected void requestRefreshPermissions()
	{
		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			Session session = Session.getActiveSession();
			if (session != null)
			{
				session.refreshPermissions();
			}
		}
	}

	protected void requestCurrentAchievements()
	{
		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			MyLog.d("FacebookBaseActivity: requestCurrentAchievements: ");

			// Create the callback
			Request.Callback callback = new Callback()
			{

				@Override
				public void onCompleted(Response response)
				{
					FacebookRequestError error = response.getError();
					if (error != null)
					{
						// We got an error
						MyLog.e("FacebookBaseActivity: requestCurrentAchievements: " + error);
					} else
					{
						try
						{
							List<Integer> ids = parseCurrentAchievementsResponse(response);
							onCurrentAchievementsResponse(ids);

						} catch (Exception e)
						{
							MyLog.e("FacebookBaseActivity: requestCurrentAchievements: Exception while parsing achievement list: "
									+ e);
						}
					}
				}
			};

			// Create a request to fetch all the user's "solve" actions
			Request request = Request.newGraphPathRequest(Session.getActiveSession(), "/me/"
					+ EARN_ACTION_TYPE, callback);
			request.executeAsync();
		}
	}

	/**
	 * Verify that we got a positive response for a Graph API publish request
	 * 
	 * @param response
	 * @return
	 */
	protected boolean verifyPublishResponse(Response response)
	{
		if (AndroidPortAdditions.SUPPORTS_FACEBOOK_ACHIEVEMENTS)
		{
			FacebookRequestError error = response.getError();
			if (error != null)
			{
				// We got an error
				MyLog.d("FacebookBaseActivity: verifyPublishResponse: " + error);

				return false;
			} else
			{
				// Check for the ID attribute in the JSON result
				String actionId = null;
				try
				{
					JSONObject graphResponse = response.getGraphObject().getInnerJSONObject();
					actionId = graphResponse.getString("id");

					MyLog.d("FacebookBaseActivity: verifyPublishResponse: success, ID is: "
							+ actionId);

				} catch (JSONException e)
				{
					MyLog.d("FacebookBaseActivity: verifyPublishResponse: " + e.getMessage());

					return false;
				}

				return true;
			}
		} else
		{
			return false;
		}
	}

	protected abstract void onSessionStateChange(Session session, SessionState state,
			Exception exception);

	protected abstract void onPublishPermissionsResponse(boolean granted);

	protected abstract void onAchievementPosted(int index, boolean explicitShare, Response response, String shareMessage);

	protected abstract void onCurrentAchievementsResponse(List<Integer> ids);

	protected abstract void onLoggedInStateChange(boolean loggedIn);

	protected Session.StatusCallback mStatusCallback = new Session.StatusCallback()
	{
		@Override
		public void call(Session session, SessionState state, Exception exception)
		{
			if (session.isOpened())
			{
				if (mLoggedInState == LoggedInState.LOGGED_OUT)
				{
					// A log in occured
					onLoggedInStateChange(true);

					StatisticsManager.getStatistics()
							.reportFacebookLogin(FacebookBaseActivity.this);
				}

				mLoggedInState = LoggedInState.LOGGED_IN;
			} else if (session.isClosed())
			{
				if (mLoggedInState == LoggedInState.LOGGED_IN)
				{
					// A log out occured
					onLoggedInStateChange(false);

					StatisticsManager.getStatistics().reportFacebookLogout(
							FacebookBaseActivity.this);

				}

				mLoggedInState = LoggedInState.LOGGED_OUT;
			}

			onSessionStateChange(session, state, exception);
		}
	};

	private List<Integer> parseCurrentAchievementsResponse(Response response) throws JSONException
	{
		List<Integer> ids = new ArrayList<Integer>();

		// Get the main JSON node
		GraphObject graphObject = response.getGraphObject();
		JSONObject jsonObject = graphObject.getInnerJSONObject();

		// Iterate over the result array
		JSONArray jsonArray = jsonObject.getJSONArray("data");
		for (int i = 0; i < jsonArray.length(); ++i)
		{
			JSONObject actionJSONObject = (JSONObject) jsonArray.get(i);
			JSONObject actionDataJSONObject = actionJSONObject.getJSONObject("data");

			try
			{
				// Get the "achievement_id" field
				int id = actionDataJSONObject.getInt(ACHIEVEMENT_ID_PROPERTY);
				// Add to the list only if it was an explicit share
				boolean explicitShare = actionDataJSONObject.getBoolean(EXPLICIT_SHARE_PROPERTY);
				if (explicitShare)
				{
					ids.add(id);
				}
			} catch (JSONException e)
			{
				// Catch the exception only for this field, but continue parsing
				MyLog.d("FacebookBaseActivity: parseCurrentAchievementsResponse: couldn't parse action field: "
						+ e);
			}
		}

		return ids;
	}

	private Session.StatusCallback mNewPermissionsCallback = new Session.StatusCallback()
	{
		@Override
		public void call(Session session, SessionState state, Exception exception)
		{
			MyLog.d("FacebookBaseActivity.mNewPermissionsCallback.new StatusCallback() {...}: call: state: "
					+ state + " exception: " + exception);

			if (exception != null)
			{
				// This means the user did not grant us write permissions
				MyLog.e("FacebookBaseActivity: call: Exception: " + exception);
				onPublishPermissionsResponse(false);
			} else if (state == SessionState.OPENED_TOKEN_UPDATED)
			{
				if (session.getPermissions().contains(PUBLISH_PERMISISONS))
				{
					// We got them
					onPublishPermissionsResponse(true);
				} else
				{
					onPublishPermissionsResponse(false);
				}
			}
		}
	};

	protected UiLifecycleHelper mFacebookUiHelper;
	
	protected String mLastShareMessage = null;

	protected static final String PUBLISH_PERMISISONS = "publish_actions";
	protected static final String ACHIEVEMENT_ID_PROPERTY = "achievement_id";
	protected static final String EXPLICIT_SHARE_PROPERTY = "explicit_share";
	protected static final String USER_MESSAGE_PROPERTY = "message";

	protected static final String ACHIEVEMENT_PROPERTY = "achievement";
	protected static final String EARN_ACTION_TYPE = "theseventhguest:earn";

	private enum LoggedInState
	{
		UNKNOWN, LOGGED_IN, LOGGED_OUT;
	}

	private LoggedInState mLoggedInState = LoggedInState.UNKNOWN;
}
